@@ -1,22 +1,35 @@
 // AlphIO.cpp
 //
-/////////////////////////////////////////////////////////////////////////////
+// Copyright (c) 2007 The Dasher Team
 //
-// Copyright (c) 2002 Iain Murray
+// This file is part of Dasher.
 //
-/////////////////////////////////////////////////////////////////////////////
+// Dasher is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
+//
+// Dasher is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Dasher; if not, write to the Free Software 
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "../../Common/Common.h"
-
 #include "AlphIO.h"
 
+#include <iostream>
+#include <cstring>
+
 using namespace Dasher;
 using namespace std;
-using namespace expat;
 
 // Track memory leaks on Windows to the line that new'd the memory
 #ifdef _WIN32
-#ifdef _DEBUG
+#ifdef _DEBUG_MEMLEAKS
 #define DEBUG_NEW new( _NORMAL_BLOCK, THIS_FILE, __LINE__ )
 #define new DEBUG_NEW
 #undef THIS_FILE
@@ -24,11 +37,11 @@ static char THIS_FILE[] = __FILE__;
 #endif
 #endif
 
-CAlphIO::CAlphIO(string SystemLocation, string UserLocation, vector <string >Filenames)
+CAlphIO::CAlphIO(std::string SystemLocation, std::string UserLocation, std::vector<std::string> &Filenames)
 :BlankInfo(), SystemLocation(SystemLocation), UserLocation(UserLocation), Filenames(Filenames), LoadMutable(false), CData("") {
   CreateDefault();
 
-  typedef pair < Opts::AlphabetTypes, string > AT;
+  typedef pair < Opts::AlphabetTypes, std::string > AT;
   vector < AT > Types;
   Types.push_back(AT(Opts::MyNone, "None"));
   Types.push_back(AT(Opts::Arabic, "Arabic"));
@@ -133,9 +146,25 @@ void CAlphIO::SetInfo(const AlphInfo &NewInfo) {
   Save(NewInfo.AlphID);
 }
 
+void CAlphIO::DeleteGroups(SGroupInfo *Info) {
+  SGroupInfo *next;
+
+  while (Info) {
+    next = Info->pNext;
+
+    if (Info->pChild)
+      DeleteGroups(Info->pChild);
+
+    delete Info;
+    Info = next;
+  }
+}
+
 void CAlphIO::Delete(const std::string &AlphID) {
-  if(Alphabets.find(AlphID) != Alphabets.end()) {
-    Alphabets.erase(AlphID);
+  map<std::string, AlphInfo>::iterator it = Alphabets.find(AlphID);
+  if(it != Alphabets.end()) {
+    DeleteGroups(it->second.m_pBaseGroup);
+    Alphabets.erase(it);
     Save("");
   }
 }
@@ -153,7 +182,7 @@ void CAlphIO::Save(const std::string &AlphID) {
   // overhead doesn't seem to matter and it makes things much easier.
   char Number[sizeof(int)];
   FILE *Output;
-  string Filename = UserLocation + "alphabet.xml";
+  std::string Filename = UserLocation + "alphabet.xml";
   if((Output = fopen(Filename.c_str(), "w")) == (FILE *) 0) {
     // could not open file
   }
@@ -314,7 +343,7 @@ void CAlphIO::CreateDefault() {
   Default.TrainingFile = "training_english_GB.txt";
   Default.GameModeFile = "gamemode_english_GB.txt";
   Default.PreferredColours = "Default";
-  string Chars = "abcdefghijklmnopqrstuvwxyz";
+  std::string Chars = "abcdefghijklmnopqrstuvwxyz";
 
 //   // Obsolete
 //   Default.Groups.resize(1);
@@ -341,7 +370,7 @@ void CAlphIO::CreateDefault() {
 void CAlphIO::XML_Escape(std::string *Text, bool Attribute) {
   // The XML "W3C Recommendation" is here: http://www.w3.org/TR/REC-xml
 
-  string & Input = *Text;       // Makes syntax less fiddly below
+  std::string & Input = *Text;       // Makes syntax less fiddly below
 
   for(unsigned int i = 0; i < Text->size(); i++) {
     // & and < need escaping in XML. In one rare circumstance >
@@ -392,6 +421,7 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     Me->InputInfo.EndConvertCharacter.Text = "";
     Me->InputInfo.m_iCharacters = 1; // Start at 1 as 0 is the root node symbol
     Me->InputInfo.m_pBaseGroup = 0;
+    Me->InputInfo.iNumChildNodes = 1; // the "root node" symbol (with text "")
     Me->bFirstGroup = true;
     Me->iGroupIdx = 0;
     while(*atts != 0) {
@@ -404,7 +434,7 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     }
     Me->InputInfo.m_iConversionID = 0;
     Me->InputInfo.m_strDefaultContext = ". ";
-
+    Me->m_vGroups.clear();
     return;
   }
 
@@ -523,8 +553,10 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
 
   if(strcmp(name, "group") == 0) {
     SGroupInfo *pNewGroup(new SGroupInfo);
+    pNewGroup->iNumChildNodes=0;
     pNewGroup->iColour = (Me->iGroupIdx % 3) + 110; 
     ++Me->iGroupIdx;
+    if (Me->m_vGroups.empty()) Me->InputInfo.iNumChildNodes++; else Me->m_vGroups.back()->iNumChildNodes++;
 
     if(Me->bFirstGroup) {
       pNewGroup->bVisible = false;
@@ -567,9 +599,9 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
 
     pNewGroup->pChild = NULL;
 
-    if(Me->InputInfo.m_vGroups.size() > 0) {
-      pNewGroup->pNext = Me->InputInfo.m_vGroups.back()->pChild;
-      Me->InputInfo.m_vGroups.back()->pChild = pNewGroup;
+    if(Me->m_vGroups.size() > 0) {
+      pNewGroup->pNext = Me->m_vGroups.back()->pChild;
+      Me->m_vGroups.back()->pChild = pNewGroup;
     }
     else {
       pNewGroup->pNext = Me->InputInfo.m_pBaseGroup;
@@ -577,7 +609,7 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     }
 
 
-    Me->InputInfo.m_vGroups.push_back(pNewGroup);
+    Me->m_vGroups.push_back(pNewGroup);
 
     return;
   }
@@ -650,6 +682,7 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     return;
   }
 
+
   if(strcmp(name, "context") == 0) {
     while(*atts != 0) {
       if(strcmp(*atts, "default") == 0) {
@@ -666,6 +699,7 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
     AlphInfo::character NewCharacter;
 
     ++Me->InputInfo.m_iCharacters;
+    if (Me->m_vGroups.empty()) Me->InputInfo.iNumChildNodes++; else Me->m_vGroups.back()->iNumChildNodes++;
 
     NewCharacter.Colour = -1;
 
@@ -704,10 +738,27 @@ void CAlphIO::XML_StartElement(void *userData, const XML_Char *name, const XML_C
   }
 }
 
+void Reverse(SGroupInfo *&pList) {
+  SGroupInfo *pFirst = pList;  
+  SGroupInfo *pPrev = NULL;
+  while (pFirst) {
+    SGroupInfo *pNext = pFirst->pNext;
+    pFirst->pNext = pPrev;
+    pPrev = pFirst;
+    pFirst = pNext;
+  }
+  pList=pPrev;
+}
+
 void CAlphIO::XML_EndElement(void *userData, const XML_Char *name) {
   CAlphIO *Me = (CAlphIO *) userData;
 
   if(strcmp(name, "alphabet") == 0) {
+    Reverse(Me->InputInfo.m_pBaseGroup);
+    if (Me->InputInfo.SpaceCharacter.Text != "") Me->InputInfo.iNumChildNodes++;
+    if (Me->InputInfo.ParagraphCharacter.Text != "") Me->InputInfo.iNumChildNodes++;
+    if (Me->InputInfo.StartConvertCharacter.Text != "") Me->InputInfo.iNumChildNodes++;
+    if (Me->InputInfo.EndConvertCharacter.Text != "") Me->InputInfo.iNumChildNodes++;
     Me->Alphabets[Me->InputInfo.AlphID] = Me->InputInfo;
     return;
   }
@@ -728,8 +779,10 @@ void CAlphIO::XML_EndElement(void *userData, const XML_Char *name) {
   }
 
   if(!strcmp(name, "group")) {
-    Me->InputInfo.m_vGroups.back()->iEnd = Me->InputInfo.m_iCharacters;
-    Me->InputInfo.m_vGroups.pop_back();
+    Me->m_vGroups.back()->iEnd = Me->InputInfo.m_iCharacters;
+    //child groups were added (to linked list) in reverse order. Put them in (iStart/iEnd) order...
+    Reverse(Me->m_vGroups.back()->pChild);
+    Me->m_vGroups.pop_back();
     return;
   }
 }
@@ -741,3 +794,8 @@ void CAlphIO::XML_CharacterData(void *userData, const XML_Char *s, int len) {
 
   Me->CData.append(s, len);
 }
+
+CAlphIO::~CAlphIO() {
+  for (std::map<std::string, AlphInfo>::iterator it = Alphabets.begin(); it != Alphabets.end(); ++it)
+    DeleteGroups(it->second.m_pBaseGroup);
+}
