@@ -18,12 +18,15 @@
 // along with Dasher; if not, write to the Free Software 
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 #include "../../Common/Common.h"
 #include "../TrainingHelper.h"
 #include "Alphabet.h"
-#include "../AlphabetManagerFactory.h"
 #include "AlphabetMap.h"
-
+#include <cstring>
+#include <sstream>
 
 using namespace Dasher;
 using namespace std;
@@ -46,8 +49,6 @@ CAlphabet::CAlphabet()
   m_Display.push_back("");
   m_Colours.push_back(-1);
   m_Foreground.push_back("");
-
-  m_pTrainingHelper = new CTrainingHelper;
 }
 
 /////////////////////////////////////////////////////////////////////////////
@@ -62,8 +63,6 @@ CAlphabet::CAlphabet(const CAlphIO::AlphInfo &AlphInfo)
   m_StartConversionSymbol = -1;
   m_EndConversionSymbol = -1;
 
-  m_pTrainingHelper = NULL;
-
   m_strDefaultContext = AlphInfo.m_strDefaultContext;
 
   // Set miscellaneous options
@@ -112,122 +111,112 @@ CAlphabet::CAlphabet(const CAlphIO::AlphInfo &AlphInfo)
   // New group stuff
 
   m_pBaseGroup = AlphInfo.m_pBaseGroup;
-
+  iNumChildNodes = AlphInfo.iNumChildNodes;
 #ifdef DASHER_TRACE
   Trace();
 #endif
 }
 
-CAlphabet::~CAlphabet() {
-  delete m_pTrainingHelper;
-}
-
 /////////////////////////////////////////////////////////////////////////////
 
-void CAlphabet::GetSymbols(std::vector<symbol > *Symbols, std::string *Input, bool IsMore) const {
-  string Tmp;
-  symbol CurSymbol = 0, TmpSymbol = 0;
-  bool KeyIsPrefix = false;
-  int extras;
-  unsigned int bit;
+CAlphabet::utf8_length::utf8_length()
+{
+  int i;
 
-  for(unsigned int i = 0; i < Input->size(); i++) {
-
-    Tmp = (*Input)[i];
-
-    /* The string we've been given is in UTF-8. The symbols are
-       also in UTF-8, so we need to pass the entire UTF-8 character
-       which may be several bytes long. RFC 2279 describes this
-       encoding */
-
-    if((*Input)[i] & 0x80) {    // Character is more than 1 byte long
-      extras = 1;
-      for(bit = 0x20; ((*Input)[i] & bit) != 0; bit >>= 1)
-        extras++;
-      if(extras > 5) {
-      }                         // Malformed character
-      while(extras-- > 0) {
-        Tmp += (*Input)[++i];
-      }
-    }
-
-    CurSymbol = TextMap.Get(Tmp, &KeyIsPrefix);
-
-    if(KeyIsPrefix) {
-      CurSymbol = 0;
-      for(; i < Input->size(); i++) {
-
-        Tmp += (*Input)[i];
-
-        TmpSymbol = TextMap.Get(Tmp, &KeyIsPrefix);
-        if(TmpSymbol > 0) {
-          CurSymbol = TmpSymbol;
-        }
-        if(!KeyIsPrefix) {
-          if(CurSymbol != 0) {
-            Symbols->push_back(CurSymbol);
-          }
-          else {
-            i -= Tmp.size() - 1;
-            //Tmp.erase(Tmp.begin(), Tmp.end());
-            Tmp = "";
-          }
-          break;
-        }
-      }
-    }
-    else {
-      if(CurSymbol != 0)
-        Symbols->push_back(CurSymbol);
-    }
-  }
-
-  if(IsMore)
-    if(KeyIsPrefix)
-      *Input = Tmp;
-    else
-      *Input = "";
-  else if(KeyIsPrefix)
-    Symbols->push_back(CurSymbol);
+  memset(utf8_count_array, 0, sizeof(utf8_count_array));
+  for (i = 0x00; i <= 0x7f; ++i) utf8_count_array[i] = 1;
+  for (i = 0xc0; i <= 0xdf; ++i) utf8_count_array[i] = 2;
+  for (i = 0xe0; i <= 0xef; ++i) utf8_count_array[i] = 3;
+  for (i = 0xf0; i <= 0xf7; ++i) utf8_count_array[i] = 4;
+  max_length = 4;
+/* The following would be valid according to RFC 2279 which was rendered
+ * obsolete by RFC 3629
+ * for (i = 0xf8; i <= 0xfb; ++i) utf8_count_array[i] = 5;
+ * for (i = 0xfc; i <= 0xfd; ++i) utf8_count_array[i] = 6;
+ * max_length = 6;
+ *
+ * and from RFC 3629:
+ * o  The octet values C0, C1, F5 to FF never appear.
+ */
+  utf8_count_array[0xc0] = utf8_count_array[0xc1] = 0;
+  for (i = 0xf5; i <= 0xff; ++i) utf8_count_array[i] = 0;
 }
 
-void CAlphabet::GetSymbolsFull(std::vector<symbol > *Symbols, std::string *Input) const {
- 
-  std::string::iterator it = Input->begin();
+CAlphabet::utf8_length CAlphabet::m_utf8_count_array;
 
-  while(it != Input->end()) {
-    unsigned char c = static_cast<unsigned char>(*it);
+int CAlphabet::utf8_length::operator[](const unsigned char i) const
+{
+  return utf8_count_array[i];
+}
 
-    int iNBytes;
+CAlphabet::SymbolStream::SymbolStream(const CAlphabet *pAlph, std::istream &_in)
+: map(pAlph->TextMap), in(_in), pos(0), len(0) {
+  readMore();
+}
 
-    if(c <= 0x7F)
-      iNBytes = 1;
-    else if((c >= 0xC2) && (c <= 0xDF))
-      iNBytes = 2;
-    else if((c >= 0xE0) && (c <= 0xEF))
-      iNBytes = 3;
-    else if((c >= 0xF0) && (c <= 0xF4))
-      iNBytes = 4;
-    else {
-      // TODO: Error condition - handle this.
-      iNBytes = 1;
-    }
-
-    std::string strCurrentSymbol(1, *it);
-
-    for(int i = 0; i < iNBytes - 1; ++i) {
-      ++it;
-      strCurrentSymbol += *it;
-    }
-
-    // TODO: Error condition on reaching end of string prematurely.
-
-    Symbols->push_back(TextMap.Get(strCurrentSymbol, NULL));
-
-    ++it;
+void CAlphabet::SymbolStream::readMore() {
+  //len is first unfilled byte
+  in.read(&buf[len], 1024-len);
+  if (in.good()) {
+    DASHER_ASSERT(in.gcount() == 1024-len);
+    len = 1024;
+  } else {
+    len+=in.gcount();
+    DASHER_ASSERT(len<1024);
+    //next attempt to read more will fail.
   }
 }
 
+symbol CAlphabet::SymbolStream::next()
+{
+  int numChars;
+
+  for (;;) {
+    if (pos + m_utf8_count_array.max_length > len && len==1024) {
+    //may need more bytes for next char; and input not yet exhausted.
+
+      if (pos) {
+        //shift remaining bytes to beginning
+        len-=pos; //len of them
+        memcpy(buf, &buf[pos], len);
+        pos=0;
+      }
+      readMore();
+    }
+  //if still don't have any chars after attempting to read more...EOF!
+    if (pos==len) return -1;
+    numChars = m_utf8_count_array[buf[pos]];
+    if (numChars != 0) break;
+#ifdef DEBUG
+    std::cerr << "Read invalid UTF-8 character 0x" << hex << buf[pos]
+              << dec << std::endl;
+#endif
+    ++pos;
+  }
+  if (numChars == 1)
+    return map.GetSingleChar(buf[pos++]);
+  if (pos+numChars > len) {
+    //no more bytes in file (would have tried to read earlier), but not enough for char
+#ifdef DEBUG
+    std::cerr << "Incomplete UTF-8 character beginning 0x" << hex << buf[pos] << dec;
+    std::cerr << "(expecting " << numChars << " bytes but only " << (len-pos) << ")" << std::endl;
+#endif
+    pos=len;
+    return -1;
+  }
+  int sym=map.Get(string(&buf[pos], numChars));
+  pos+=numChars;
+  return sym;
+}
+
+void CAlphabet::GetSymbols(std::vector<symbol>& Symbols, const std::string& Input) const
+{
+  std::istringstream in(Input);
+  SymbolStream syms(this, in);
+  for (symbol sym; (sym=syms.next())!=-1;)
+    Symbols.push_back(sym);
+}
+
 // add single char to the character set
 void CAlphabet::AddChar(const std::string NewCharacter, const std::string Display, int Colour, const std::string Foreground) {
   m_Characters.push_back(NewCharacter);
@@ -311,27 +300,6 @@ int CAlphabet::GetTextColour(symbol Symbol) {
   }
 }
 
-
-void 
-CAlphabet::Train(const std::string &strUserLoc, 
-		 const std::string &strSystemLoc, 
-		 CTrainer *pTrainer) {
-
-  std::string strTrainingFile = GetTrainingFile();
-
-  std::string strUserPathFull = strUserLoc + strTrainingFile;
-
-  m_pTrainingHelper->LoadFile(strUserPathFull, pTrainer, this);
-  m_pTrainingHelper->LoadFile(strSystemLoc + strTrainingFile, pTrainer, this);
-}
-
-void 
-CAlphabet::Train(const std::string &strPath, 
-      CTrainer *pTrainer) {
-  
-  m_pTrainingHelper->LoadFile(strPath, pTrainer, this);
-}
-
 int 
 CAlphabet::GetColour(symbol i, int iPhase) const {
   int iColour = m_Colours[i];
