@@ -2,7 +2,8 @@
 #define __GROUP_INFO_H__
 
 #include <string>
-
+/// \ingroup Alphabet
+/// \{
 struct SGroupInfo {
   SGroupInfo *pChild;
   SGroupInfo *pNext;
@@ -11,6 +12,8 @@ struct SGroupInfo {
   int iEnd;
   int iColour;
   bool bVisible;
+  int iNumChildNodes;
 };
+/// \}
 
 #endif
