@@ -3,6 +3,8 @@
 #include "DasherInterfaceBase.h"
 #include "Event.h"
 
+using namespace Dasher;
+
 static SModuleSettings sSettings[] = {
   {LP_MAXZOOM, T_LONG, 11, 400, 10, 1, _("Maximum Zoom")},
 };
@@ -11,8 +13,8 @@ bool CClickFilter::DecorateView(CDasherView *pView) {
   return false;
 }
 
-bool CClickFilter::Timer(int Time, CDasherView *pDasherView, CDasherModel *pModel, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted) {
-  return pModel->UpdatePosition(0, 0, Time, pAdded, pNumDeleted);
+bool CClickFilter::Timer(int Time, CDasherView *pDasherView, CDasherModel *pModel, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted, CExpansionPolicy **pol) {
+  return pModel->NextScheduledStep(Time, pAdded, pNumDeleted);
 }
 
 void CClickFilter::KeyDown(int iTime, int iId, CDasherView *pDasherView, CDasherModel *pModel, CUserLogBase *pUserLog, bool bPos, int iX, int iY) {
