@@ -23,7 +23,7 @@ static char THIS_FILE[] = __FILE__;
 
 // TODO: Share information with AlphIO class?
 
-CColourIO::CColourIO(std::string SystemLocation, std::string UserLocation, std::vector<std::string> Filenames)
+CColourIO::CColourIO(std::string SystemLocation, std::string UserLocation, std::vector<std::string> &Filenames)
 :BlankInfo(), SystemLocation(SystemLocation), UserLocation(UserLocation), Filenames(Filenames), LoadMutable(false), CData("") {
   CreateDefault();
 
