@@ -21,6 +21,8 @@
 #include "../Common/Common.h"
 
 #include "ControlManager.h"
+#include <cstring>
+
 using namespace Dasher;
 using namespace std;
 // Track memory leaks on Windows to the line that new'd the memory
@@ -36,7 +38,7 @@ static char THIS_FILE[] = __FILE__;
 int CControlManager::m_iNextID = 0;
 
 CControlManager::CControlManager( CNodeCreationManager *pNCManager )
-  : CNodeManager(1), m_pNCManager(pNCManager), m_pLanguageModel(NULL) {
+  : m_pNCManager(pNCManager) {
   string SystemString = m_pNCManager->GetStringParameter(SP_SYSTEM_LOC);
   string UserLocation = m_pNCManager->GetStringParameter(SP_USER_LOC);
   m_iNextID = 0;
@@ -222,8 +224,8 @@ int CControlManager::ConnectNodes() {
 
 CControlManager::~CControlManager()
 {
-  for(std::map<int,CControlNode*>::iterator i = m_mapControlMap.begin(); i != m_mapControlMap.end(); i++) {
-    CControlNode* pNewNode = i->second;
+  for(std::map<int,SControlItem*>::iterator i = m_mapControlMap.begin(); i != m_mapControlMap.end(); i++) {
+    SControlItem* pNewNode = i->second;
     if (pNewNode != NULL) {
       delete pNewNode;
       pNewNode = NULL;
@@ -232,9 +234,9 @@ CControlManager::~CControlManager()
 }
 
 void CControlManager::RegisterNode( int iID, std::string strLabel, int iColour ) {
-  CControlNode *pNewNode;
+  SControlItem *pNewNode;
   
-  pNewNode = new CControlNode; // FIXME - do constructor sanely
+  pNewNode = new SControlItem; // FIXME - do constructor sanely
   pNewNode->strLabel = strLabel;
   pNewNode->iID = iID;
   pNewNode->iColour = iColour;
@@ -247,7 +249,7 @@ void CControlManager::ConnectNode(int iChild, int iParent, int iAfter) {
   // FIXME - iAfter currently ignored (eventually -1 = start, -2 = end)
 
   if( iChild == -1 ) {// Corresponds to escaping back to alphabet
-    CControlNode* node = m_mapControlMap[iParent];
+    SControlItem* node = m_mapControlMap[iParent];
     if(node)
       node->vChildren.push_back(NULL);
   }
@@ -256,17 +258,16 @@ void CControlManager::ConnectNode(int iChild, int iParent, int iAfter) {
 }
 
 void CControlManager::DisconnectNode(int iChild, int iParent) {
-  CControlNode* pParentNode = m_mapControlMap[iParent];
-  CControlNode* pChildNode = m_mapControlMap[iChild];
+  SControlItem* pParentNode = m_mapControlMap[iParent];
+  SControlItem* pChildNode = m_mapControlMap[iChild];
 
-  for(std::vector<CControlNode *>::iterator itChild(pParentNode->vChildren.begin()); itChild != pParentNode->vChildren.end(); ++itChild)
+  for(std::vector<SControlItem *>::iterator itChild(pParentNode->vChildren.begin()); itChild != pParentNode->vChildren.end(); ++itChild)
     if(*itChild == pChildNode)
       pParentNode->vChildren.erase(itChild);
 }
 
 
-CDasherNode *CControlManager::GetRoot(CDasherNode *pParent, int iLower, int iUpper, void *pUserData) {
-  CDasherNode *pNewNode;
+CDasherNode *CControlManager::GetRoot(CDasherNode *pParent, int iLower, int iUpper, int iOffset) {
 
   // TODO: Tie this structure to info contained in control map
   CDasherNode::SDisplayInfo *pDisplayInfo = new CDasherNode::SDisplayInfo;
@@ -275,53 +276,42 @@ CDasherNode *CControlManager::GetRoot(CDasherNode *pParent, int iLower, int iUpp
   pDisplayInfo->bVisible = true;
   pDisplayInfo->strDisplayText = m_mapControlMap[0]->strLabel;
   
-  pNewNode = new CDasherNode(pParent, iLower, iUpper, pDisplayInfo);
-
-  int iOffset = *((int *)pUserData);
+  CContNode *pNewNode = new CContNode(pParent, iLower, iUpper, pDisplayInfo, this);
  
   // FIXME - handle context properly
 
   //  pNewNode->SetContext(m_pLanguageModel->CreateEmptyContext());
 
-  pNewNode->m_pNodeManager = this;
-
-  SControlData *pNodeUserData = new SControlData;
-
-  pNodeUserData->pControlNode = m_mapControlMap[0];
-  pNodeUserData->iOffset = iOffset;
-
-  pNewNode->m_pUserData = pNodeUserData;
+  pNewNode->pControlItem = m_mapControlMap[0];
+  pNewNode->m_iOffset = iOffset;
 
   return pNewNode;
 }
 
-void CControlManager::PopulateChildren( CDasherNode *pNode ) {
-   CDasherNode *pNewNode;
+CControlManager::CContNode::CContNode(CDasherNode *pParent, int iLbnd, int iHbnd, CDasherNode::SDisplayInfo *pDisplayInfo, CControlManager *pMgr)
+: CDasherNode(pParent, iLbnd, iHbnd, pDisplayInfo), m_pMgr(pMgr) {
+}
 
-   CControlNode *pControlNode((static_cast<SControlData *>(pNode->m_pUserData))->pControlNode);
 
-   int iNChildren( pControlNode->vChildren.size() );
+void CControlManager::CContNode::PopulateChildren() {
+  
+  CDasherNode *pNewNode;
+
+   int iNChildren( pControlItem->vChildren.size() );
 
    int iIdx(0);
 
-   for(std::vector<CControlNode *>::iterator it(pControlNode->vChildren.begin()); it != pControlNode->vChildren.end(); ++it) {
+   for(std::vector<SControlItem *>::iterator it(pControlItem->vChildren.begin()); it != pControlItem->vChildren.end(); ++it) {
 
      // FIXME - could do this better
 
-     int iLbnd( iIdx*(m_pNCManager->GetLongParameter(LP_NORMALIZATION)/iNChildren)); 
-     int iHbnd( (iIdx+1)*(m_pNCManager->GetLongParameter(LP_NORMALIZATION)/iNChildren)); 
+     int iLbnd( iIdx*(m_pMgr->m_pNCManager->GetLongParameter(LP_NORMALIZATION)/iNChildren)); 
+     int iHbnd( (iIdx+1)*(m_pMgr->m_pNCManager->GetLongParameter(LP_NORMALIZATION)/iNChildren)); 
 
      if( *it == NULL ) {
        // Escape back to alphabet
-       CAlphabetManager::SRootData *pRootData = new CAlphabetManager::SRootData;
 
-       // TODO: Check that these are eventually getting deleted
-
-       pRootData->iOffset = (static_cast<SControlData *>(pNode->m_pUserData))->iOffset;
-       pRootData->szContext = NULL; // TODO: Fix this
-
-       pNewNode = m_pNCManager->GetRoot(0, pNode, iLbnd, iHbnd, pRootData);
-       pNewNode->SetFlag(NF_SEEN, false);
+       pNewNode = m_pMgr->m_pNCManager->GetAlphRoot(this, iLbnd, iHbnd, false, m_iOffset);
      }
      else {
 
@@ -337,66 +327,64 @@ void CControlManager::PopulateChildren( CDasherNode *pNode ) {
        pDisplayInfo->bVisible = true;
        pDisplayInfo->strDisplayText = (*it)->strLabel;
        
-       pNewNode = new CDasherNode(pNode, iLbnd, iHbnd, pDisplayInfo);
+       CContNode *pContNode;
+       pNewNode = pContNode = new CContNode(this, iLbnd, iHbnd, pDisplayInfo, m_pMgr);
 
-       pNewNode->m_pNodeManager = this;
-       pNewNode->m_pUserData = *it;
-
-       SControlData *pNodeUserData = new SControlData;
-
-       pNodeUserData->pControlNode = *it;
-       pNodeUserData->iOffset = (static_cast<SControlData *>(pNode->m_pUserData))->iOffset;
-
-       pNewNode->m_pUserData = pNodeUserData;
+       pContNode->pControlItem = *it;
 
+       pNewNode->m_iOffset = m_iOffset;
      }
-     pNode->Children().push_back(pNewNode);
+     DASHER_ASSERT(GetChildren().back()==pNewNode);
      ++iIdx;
    }
 }
-
-void CControlManager::ClearNode( CDasherNode *pNode ) {
-  delete (static_cast<SControlData *>(pNode->m_pUserData));
+int CControlManager::CContNode::ExpectedNumChildren() {
+  return pControlItem->vChildren.size();
 }
+void CControlManager::CContNode::Output(Dasher::VECTOR_SYMBOL_PROB* pAdded, int iNormalization ) {
 
-void CControlManager::Output( CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* pAdded, int iNormalization ) {
-
-  CControlNode *pControlNode((static_cast<SControlData *>(pNode->m_pUserData))->pControlNode);
-
-  CControlEvent oEvent(pControlNode->iID);
+  CControlEvent oEvent(pControlItem->iID);
   // TODO: Need to reimplement this
   //  m_pNCManager->m_bContextSensitive=false;
-  m_pNCManager->InsertEvent(&oEvent);
+  m_pMgr->m_pNCManager->InsertEvent(&oEvent);
 }
 
-void CControlManager::Undo( CDasherNode *pNode ) {
+void CControlManager::CContNode::Undo() {
   // Do we ever need this?
   // One other thing we probably want is notification when we leave a node - that way we can eg speed up again if we slowed down
-  m_pNCManager->SetLongParameter(LP_SPEED_DIVISOR, 100);
+  m_pMgr->m_pNCManager->SetLongParameter(LP_BOOSTFACTOR, 100);
   //Re-enable auto speed control!
-  m_pNCManager->SetBoolParameter(BP_AUTO_SPEEDCONTROL, 1);
-  
+  if (m_pMgr->bDisabledSpeedControl)
+  {
+    m_pMgr->bDisabledSpeedControl = false;
+    m_pMgr->m_pNCManager->SetBoolParameter(BP_AUTO_SPEEDCONTROL, 1);
+  }
 }
 
-void CControlManager::Enter(CDasherNode *pNode) {
+void CControlManager::CContNode::Enter() {
   // Slow down to half the speed we were at
-  m_pNCManager->SetLongParameter(LP_SPEED_DIVISOR, 200);
+  m_pMgr->m_pNCManager->SetLongParameter(LP_BOOSTFACTOR, 50);
   //Disable auto speed control!
-  m_pNCManager->SetBoolParameter(BP_AUTO_SPEEDCONTROL, 0);
+  m_pMgr->bDisabledSpeedControl = m_pMgr->m_pNCManager->GetBoolParameter(BP_AUTO_SPEEDCONTROL); 
+  m_pMgr->m_pNCManager->SetBoolParameter(BP_AUTO_SPEEDCONTROL, 0);
 }
 
 
-void CControlManager::Leave(CDasherNode *pNode) {
+void CControlManager::CContNode::Leave() {
   // Now speed back up, by doubling the speed we were at in control mode
-  m_pNCManager->SetLongParameter(LP_SPEED_DIVISOR, 100);
+  m_pMgr->m_pNCManager->SetLongParameter(LP_BOOSTFACTOR, 100);
   //Re-enable auto speed control!
-  m_pNCManager->SetBoolParameter(BP_AUTO_SPEEDCONTROL, 1);
+  if (m_pMgr->bDisabledSpeedControl)
+  {
+    m_pMgr->bDisabledSpeedControl = false;
+    m_pMgr->m_pNCManager->SetBoolParameter(BP_AUTO_SPEEDCONTROL, 1);
+  }
 }
 
 
 void CControlManager::XmlStartHandler(void *pUserData, const XML_Char *szName, const XML_Char **aszAttr) {
   
-  int colour;
+  int colour=-1;
   string str;
   if(0==strcmp(szName, "label"))
   {
@@ -424,6 +412,6 @@ void CControlManager::XmlCDataHandler(void *pUserData, const XML_Char *szData, i
   return;
 }
 
-void CControlManager::SetControlOffset(CDasherNode *pNode, int iOffset) {
-  (static_cast<SControlData *>(pNode->m_pUserData))->iOffset = iOffset;
+void CControlManager::CContNode::SetControlOffset(int iOffset) {
+  m_iOffset = iOffset;
 }
