@@ -21,8 +21,6 @@
 #ifndef __controlmanager_h__
 #define __controlmanager_h__
 
-#include "NodeManager.h"
-#include "LanguageModelling/LanguageModel.h" // Urgh - we really shouldn't need to know about language models here
 #include "DasherModel.h"
 #include "DasherNode.h"
 #include "Event.h"
@@ -53,17 +51,9 @@ namespace Dasher {
   /// Currently can only have one instance due to use 
   /// of static members for callbacks from expat.
   ///
-  class CControlManager : public CNodeManager {
+  class CControlManager {
   public:
 
-    class CControlNode {
-    public: // Worry about encapsulation later
-      std::vector<CControlNode *> vChildren;
-      std::string strLabel;
-      int iID;
-      int iColour;
-    };
-
     enum { CTL_ROOT, CTL_STOP, CTL_PAUSE, CTL_MOVE, CTL_MOVE_FORWARD, 
 	   CTL_MOVE_FORWARD_CHAR, CTL_MOVE_FORWARD_WORD, CTL_MOVE_FORWARD_LINE,
 	   CTL_MOVE_FORWARD_FILE, CTL_MOVE_BACKWARD, CTL_MOVE_BACKWARD_CHAR, 
@@ -78,50 +68,47 @@ namespace Dasher {
     CControlManager(CNodeCreationManager *pNCManager);
     ~CControlManager();
 
-    ///
-    /// Does nothing - control manager isn't reference counted.
-    ///
-
-    virtual void Ref() {};
-    
-    ///
-    /// Does nothing - control manager isn't reference counted.
-    ///
-    
-    virtual void Unref() {};
-
     ///
     /// Get a new root node owned by this manager
     ///
 
-    virtual CDasherNode *GetRoot(CDasherNode *pParent, int iLower, int iUpper, void *pUserData);
+    virtual CDasherNode *GetRoot(CDasherNode *pParent, int iLower, int iUpper, int iOffset);
+    void RegisterNode( int iID, std::string strLabel, int iColour );
+    void ConnectNode(int iChild, int iParent, int iAfter);
+    void DisconnectNode(int iChild, int iParent);
+    
+  private:
 
+    struct SControlItem {
+      std::vector<SControlItem *> vChildren;
+      std::string strLabel;
+      int iID;
+      int iColour;
+    };
+    
+    class CContNode : public CDasherNode {
+    public:
+      int mgrId() {return 1;}
+      CContNode(CDasherNode *pParent, int iLbnd, int iHbnd, CDasherNode::SDisplayInfo *pDisplayInfo, CControlManager *pMgr);
     ///
     /// Provide children for the supplied node
     ///
 
-    virtual void PopulateChildren( CDasherNode *pNode );
+    virtual void PopulateChildren();
+      virtual int ExpectedNumChildren();
     
-    ///
-    /// Delete any storage alocated for this node
-    ///
+    virtual void Output(Dasher::VECTOR_SYMBOL_PROB* pAdded, int iNormalization );
+    virtual void Undo();
 
-    virtual void ClearNode( CDasherNode *pNode );
+    virtual void Enter();
+    virtual void Leave();
+
+    void SetControlOffset(int iOffset);
+      SControlItem *pControlItem;
+    private:
+      CControlManager *m_pMgr;
+    };
     
-    virtual void Output( CDasherNode *pNode, Dasher::VECTOR_SYMBOL_PROB* pAdded, int iNormalization );
-    virtual void Undo( CDasherNode *pNode );
-
-    virtual void Enter(CDasherNode *pNode);
-    virtual void Leave(CDasherNode *pNode);
-
-    void RegisterNode( int iID, std::string strLabel, int iColour );
-    void ConnectNode(int iChild, int iParent, int iAfter);
-    void DisconnectNode(int iChild, int iParent);
-
-    void SetControlOffset(CDasherNode *pNode, int iOffset);
-
-  private:
-
     static void XmlStartHandler(void *pUserData, const XML_Char *szName, const XML_Char **aszAttr);
     static void XmlEndHandler(void *pUserData, const XML_Char *szName);
     static void XmlCDataHandler(void *pUserData, const XML_Char *szData, int iLength);
@@ -132,13 +119,12 @@ namespace Dasher {
 
     static int m_iNextID;
     CNodeCreationManager *m_pNCManager;
-    CLanguageModel *m_pLanguageModel;
-    std::map<int,CControlNode*> m_mapControlMap;
+    std::map<int,SControlItem*> m_mapControlMap;
+
+    ///Whether we'd temporarily disabled Automatic Speed Control
+    ///(if _and only if_ so, should re-enable it when leaving a node)
+    bool bDisabledSpeedControl;
 
-    struct SControlData {
-      CControlNode *pControlNode;
-      int iOffset;
-    };
   };
   /// @}
 }
