@@ -1,6 +1,6 @@
 // DasherInterfaceBase.cpp
 //
-// Copyright (c) 2007 The Dasher Team
+// Copyright (c) 2008 The Dasher Team
 //
 // This file is part of Dasher.
 //
@@ -15,7 +15,7 @@
 // GNU General Public License for more details.
 //
 // You should have received a copy of the GNU General Public License
-// along with Dasher; if not, write to the Free Software 
+// along with Dasher; if not, write to the Free Software
 // Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "../Common/Common.h"
@@ -23,7 +23,6 @@
 #include "DasherInterfaceBase.h"
 
 //#include "ActionButton.h"
-#include "AlphabetManagerFactory.h"
 #include "DasherViewSquare.h"
 #include "ControlManager.h"
 #include "DasherScreen.h"
@@ -33,39 +32,45 @@
 #include "EventHandler.h"
 #include "Event.h"
 #include "NodeCreationManager.h"
+#ifndef _WIN32_WCE
 #include "UserLog.h"
 #include "BasicLog.h"
-#include "WrapperFactory.h"
+#endif
+#include "DasherGameMode.h"
 
 // Input filters
-#include "ClickFilter.h" 
+#include "AlternatingDirectMode.h"
+#include "ButtonMode.h"
+#include "ClickFilter.h"
+#include "CompassMode.h"
 #include "DefaultFilter.h"
-#include "DasherButtons.h"
-#include "DynamicFilter.h"
+
 #include "EyetrackerFilter.h"
+#include "OneButtonFilter.h"
+#include "OneButtonDynamicFilter.h"
 #include "OneDimensionalFilter.h"
 #include "StylusFilter.h"
 #include "TwoButtonDynamicFilter.h"
+#include "TwoPushDynamicFilter.h"
 
 // STL headers
+#include <cstdio>
 #include <iostream>
 #include <memory>
 
-// Legacy C library headers
-namespace {
-  #include "stdio.h"
-}
-
 // Declare our global file logging object
 #include "../DasherCore/FileLogger.h"
 #ifdef _DEBUG
 const eLogLevel g_iLogLevel   = logDEBUG;
-const int       g_iLogOptions = logTimeStamp | logDateStamp | logDeleteOldFile;    
+const int       g_iLogOptions = logTimeStamp | logDateStamp | logDeleteOldFile;
 #else
 const eLogLevel g_iLogLevel   = logNORMAL;
 const int       g_iLogOptions = logTimeStamp | logDateStamp;
 #endif
+
+#ifndef _WIN32_WCE
 CFileLogger* g_pLogger = NULL;
+#endif
 
 using namespace Dasher;
 using namespace std;
@@ -88,19 +93,19 @@ CDasherInterfaceBase::CDasherInterfaceBase() {
   m_DasherScreen = NULL;
   m_pDasherView = NULL;
   m_pInput = NULL;
+  m_pInputFilter = NULL;
   m_AlphIO = NULL;
   m_ColourIO = NULL;
   m_pUserLog = NULL;
-  m_pInputFilter = NULL;
   m_pNCManager = NULL;
+  m_defaultPolicy = NULL;
 
   // Various state variables
   m_bRedrawScheduled = false;
-  
+
   m_iCurrentState = ST_START;
-  
-  m_iLockCount = 0;
-  m_bGlobalLock = false;
+
+  //  m_bGlobalLock = false;
 
   // TODO: Are these actually needed?
   strCurrentContext = ". ";
@@ -111,15 +116,18 @@ CDasherInterfaceBase::CDasherInterfaceBase() {
 
   m_bLastChanged = true;
 
+#ifndef _WIN32_WCE
   // Global logging object we can use from anywhere
   g_pLogger = new CFileLogger("dasher.log",
                               g_iLogLevel,
                               g_iLogOptions);
+#endif
+
+  m_iNextLockID = 1;
 
 }
 
 void CDasherInterfaceBase::Realize() {
-
   // TODO: What exactly needs to have happened by the time we call Realize()?
   CreateSettingsStore();
   SetupUI();
@@ -134,7 +142,7 @@ void CDasherInterfaceBase::Realize() {
   m_ColourIO = new CColourIO(GetStringParameter(SP_SYSTEM_LOC), GetStringParameter(SP_USER_LOC), vColourFiles);
 
   ChangeColours();
-  ChangeAlphabet();
+  ChangeAlphabet(); // This creates the NodeCreationManager, the Alphabet
 
   // Create the user logging object if we are suppose to.  We wait
   // until now so we have the real value of the parameter and not
@@ -142,45 +150,69 @@ void CDasherInterfaceBase::Realize() {
 
   // TODO: Sort out log type selection
 
+#ifndef _WIN32_WCE
   int iUserLogLevel = GetLongParameter(LP_USER_LOG_LEVEL_MASK);
 
   if(iUserLogLevel == 10)
     m_pUserLog = new CBasicLog(m_pEventHandler, m_pSettingsStore);
-  else if (iUserLogLevel > 0) 
-    m_pUserLog = new CUserLog(m_pEventHandler, m_pSettingsStore, iUserLogLevel, m_Alphabet);  
+  else if (iUserLogLevel > 0)
+    m_pUserLog = new CUserLog(m_pEventHandler, m_pSettingsStore, iUserLogLevel, m_Alphabet);
+#else
+  m_pUserLog = NULL;
+#endif
 
-  CreateFactories();
-  CreateLocalFactories();
+  CreateModules();
 
   CreateInput();
   CreateInputFilter();
   SetupActionButtons();
+  CParameterNotificationEvent oEvent(LP_NODE_BUDGET);
+  InterfaceEventHandler(&oEvent);
+
+  // Set up real orientation to match selection
+  if(GetLongParameter(LP_ORIENTATION) == Dasher::Opts::AlphabetDefault)
+    SetLongParameter(LP_REAL_ORIENTATION, m_Alphabet->GetOrientation());
+  else
+    SetLongParameter(LP_REAL_ORIENTATION, GetLongParameter(LP_ORIENTATION));
+
 
   // FIXME - need to rationalise this sort of thing.
   // InvalidateContext(true);
   ScheduleRedraw();
-    
+
+#ifndef _WIN32_WCE
   // All the setup is done by now, so let the user log object know
   // that future parameter changes should be logged.
-  if (m_pUserLog != NULL) 
+  if (m_pUserLog != NULL)
     m_pUserLog->InitIsDone();
+#endif
 
   // TODO: Make things work when model is created latet
   ChangeState(TR_MODEL_INIT);
+
+  using GameMode::CDasherGameMode;
+  // Create the teacher singleton object.
+  CDasherGameMode::CreateTeacher(m_pEventHandler, m_pSettingsStore, this);
+  CDasherGameMode::GetTeacher()->SetDasherView(m_pDasherView);
+  CDasherGameMode::GetTeacher()->SetDasherModel(m_pDasherModel);
 }
 
 CDasherInterfaceBase::~CDasherInterfaceBase() {
   DASHER_ASSERT(m_iCurrentState == ST_SHUTDOWN);
 
+  // It may seem odd that InterfaceBase does not "own" the teacher.
+  // This is because game mode is a different layer, in a sense.
+  GameMode::CDasherGameMode::DestroyTeacher();
+
   delete m_pDasherModel;        // The order of some of these deletions matters
   delete m_Alphabet;
   delete m_pDasherView;
   delete m_ColourIO;
   delete m_AlphIO;
-  delete m_pInputFilter;
   delete m_pNCManager;
   // Do NOT delete Edit box or Screen. This class did not create them.
 
+#ifndef _WIN32_WCE
   // When we destruct on shutdown, we'll output any detailed log file
   if (m_pUserLog != NULL)
   {
@@ -193,6 +225,14 @@ CDasherInterfaceBase::~CDasherInterfaceBase() {
     delete g_pLogger;
     g_pLogger = NULL;
   }
+#endif
+
+
+  for (std::vector<CActionButton *>::iterator it=m_vLeftButtons.begin(); it != m_vLeftButtons.end(); ++it)
+    delete *it;
+
+  for (std::vector<CActionButton *>::iterator it=m_vRightButtons.begin(); it != m_vRightButtons.end(); ++it)
+    delete *it;
 
   // Must delete event handler after all CDasherComponent derived classes
 
@@ -205,20 +245,20 @@ void CDasherInterfaceBase::PreSetNotify(int iParameter, const std::string &sNewV
   // infrastructure
 
   switch(iParameter) {
-  case SP_ALPHABET_ID: 
+  case SP_ALPHABET_ID:
     // Cycle the alphabet history
     if(GetStringParameter(SP_ALPHABET_ID) != sNewValue) {
       if(GetStringParameter(SP_ALPHABET_1) != sNewValue) {
 	if(GetStringParameter(SP_ALPHABET_2) != sNewValue) {
 	  if(GetStringParameter(SP_ALPHABET_3) != sNewValue)
 	    SetStringParameter(SP_ALPHABET_4, GetStringParameter(SP_ALPHABET_3));
-	  
+
 	  SetStringParameter(SP_ALPHABET_3, GetStringParameter(SP_ALPHABET_2));
 	}
-	
+
 	SetStringParameter(SP_ALPHABET_2, GetStringParameter(SP_ALPHABET_1));
       }
-      
+
       SetStringParameter(SP_ALPHABET_1, GetStringParameter(SP_ALPHABET_ID));
     }
 
@@ -228,7 +268,7 @@ void CDasherInterfaceBase::PreSetNotify(int iParameter, const std::string &sNewV
 
 void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
 
-  if(pEvent->m_iEventType == 1) {
+  if(pEvent->m_iEventType == EV_PARAM_NOTIFY) {
     Dasher::CParameterNotificationEvent * pEvt(static_cast < Dasher::CParameterNotificationEvent * >(pEvent));
 
     switch (pEvt->m_iParameter) {
@@ -262,7 +302,7 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
       ScheduleRedraw();
       break;
     case SP_DEFAULT_COLOUR_ID: // Delibarate fallthrough
-    case BP_PALETTE_CHANGE: 
+    case BP_PALETTE_CHANGE:
       if(GetBoolParameter(BP_PALETTE_CHANGE))
 	 SetStringParameter(SP_COLOUR_ID, GetStringParameter(SP_DEFAULT_COLOUR_ID));
       break;
@@ -285,24 +325,30 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
     case BP_DASHER_PAUSED:
       ScheduleRedraw();
       break;
+    case LP_MARGIN_WIDTH:
+        ScheduleRedraw();
+        break;
+    case LP_NODE_BUDGET:
+      delete m_defaultPolicy;
+      m_defaultPolicy = new AmortizedPolicy(GetLongParameter(LP_NODE_BUDGET));  
     default:
       break;
     }
   }
-  else if(pEvent->m_iEventType == 2) {
+  else if(pEvent->m_iEventType == EV_EDIT && !GetBoolParameter(BP_GAME_MODE)) {
     CEditEvent *pEditEvent(static_cast < CEditEvent * >(pEvent));
-    
+
     if(pEditEvent->m_iEditType == 1) {
       strCurrentContext += pEditEvent->m_sText;
       if( strCurrentContext.size() > 20 )
 	strCurrentContext = strCurrentContext.substr( strCurrentContext.size() - 20 );
-
-      strTrainfileBuffer += pEditEvent->m_sText;
+      if(GetBoolParameter(BP_LM_ADAPTIVE))
+	 strTrainfileBuffer += pEditEvent->m_sText;
     }
     else if(pEditEvent->m_iEditType == 2) {
       strCurrentContext = strCurrentContext.substr( 0, strCurrentContext.size() - pEditEvent->m_sText.size());
-
-      strTrainfileBuffer = strTrainfileBuffer.substr( 0, strTrainfileBuffer.size() - pEditEvent->m_sText.size());
+      if(GetBoolParameter(BP_LM_ADAPTIVE))
+	 strTrainfileBuffer = strTrainfileBuffer.substr( 0, strTrainfileBuffer.size() - pEditEvent->m_sText.size());
     }
   }
   else if(pEvent->m_iEventType == EV_CONTROL) {
@@ -313,24 +359,11 @@ void CDasherInterfaceBase::InterfaceEventHandler(Dasher::CEvent *pEvent) {
       PauseAt(0,0);
       break;
     case CControlManager::CTL_PAUSE:
-      Halt();
-      break;
-    }
-  }
-  else if(pEvent->m_iEventType == EV_LOCK) {
-    CLockEvent *pLockEvent(static_cast<CLockEvent *>(pEvent));
+  //Halt Dasher - without a stop event, so does not result in speech etc.
+      SetBoolParameter(BP_DASHER_PAUSED, true);
 
-//     // TODO: Sort this out - at the moment these don't occur in pairs, so the old boolean variable is still needed
-//     if(pLockEvent->m_bLock) {
-//       if(m_bGlobalLock)
-// 	AddLock(0);
-//     }
-//     else {
-//       if(!m_bGlobalLock)
-// 	ReleaseLock(0);
-//     }
-    
-    m_bGlobalLock = pLockEvent->m_bLock;
+      m_pDasherModel->TriggerSlowdown();
+    }
   }
 }
 
@@ -354,13 +387,13 @@ void CDasherInterfaceBase::CreateModel(int iOffset) {
     delete m_pDasherModel;
     m_pDasherModel = 0;
   }
-  
-  if(m_deGameModeStrings.size() == 0) {
-    m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, m_pNCManager, this, m_pDasherView, iOffset);
-  }
-  else {
-    m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, m_pNCManager, this, m_pDasherView, iOffset, true, m_deGameModeStrings[0]);
-  }
+
+  m_pDasherModel = new CDasherModel(m_pEventHandler, m_pSettingsStore, m_pNCManager, this, m_pDasherView, iOffset);
+
+  // Notify the teacher of the new model
+  if(GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher())
+    pTeacher->SetDasherModel(m_pDasherModel);
+
 }
 
 void CDasherInterfaceBase::CreateNCManager() {
@@ -371,18 +404,9 @@ void CDasherInterfaceBase::CreateNCManager() {
 
   int lmID = GetLongParameter(LP_LANGUAGE_MODEL_ID);
 
-  if( lmID == -1 ) 
+  if( lmID == -1 )
     return;
 
- // Train the new language model
-    CLockEvent *pEvent;
-    
-    pEvent = new CLockEvent("Training Dasher", true, 0);
-    m_pEventHandler->InsertEvent(pEvent);
-    delete pEvent;
-
-    AddLock(0);
-
     int iOffset;
 
     if(m_pDasherModel)
@@ -400,65 +424,15 @@ void CDasherInterfaceBase::CreateNCManager() {
       delete m_pNCManager;
       m_pNCManager = 0;
     }
-      
-    if(m_deGameModeStrings.size() == 0) {
-      m_pNCManager = new CNodeCreationManager(this, m_pEventHandler, m_pSettingsStore, false, "", m_AlphIO);
-    }
-    else {
-      m_pNCManager = new CNodeCreationManager(this, m_pEventHandler, m_pSettingsStore, true, m_deGameModeStrings[0], m_AlphIO);
-    }
+
+    m_pNCManager = new CNodeCreationManager(this, m_pEventHandler, m_pSettingsStore, m_AlphIO);
 
     m_Alphabet = m_pNCManager->GetAlphabet();
-    
-    string T = m_Alphabet->GetTrainingFile();
-
-    int iTotalBytes = 0;
-    iTotalBytes += GetFileSize(GetStringParameter(SP_SYSTEM_LOC) + T);
-    iTotalBytes += GetFileSize(GetStringParameter(SP_USER_LOC) + T);
-
-    if(iTotalBytes > 0) {
-      int iOffset;
-      iOffset = TrainFile(GetStringParameter(SP_SYSTEM_LOC) + T, iTotalBytes, 0);
-      TrainFile(GetStringParameter(SP_USER_LOC) + T, iTotalBytes, iOffset);
-    }
-    else {
-      CMessageEvent oEvent("No training text is avilable for the selected alphabet. Dasher will function, but it may be difficult to enter text.\nPlease see http://www.dasher.org.uk/alphabets/ for more information.", 0, 0);
-      m_pEventHandler->InsertEvent(&oEvent);
-    }
-
-    pEvent = new CLockEvent("Training Dasher", false, 0);
-    m_pEventHandler->InsertEvent(pEvent);
-    delete pEvent;
-
-    ReleaseLock(0);
 
     // TODO: Eventually we'll not have to pass the NC manager to the model...
     CreateModel(iOffset);
 }
 
-// void CDasherInterfaceBase::Start() {
-//   // TODO: Clarify the relationship between Start() and
-//   // InvalidateContext() - I believe that they essentially do the same
-//   // thing
-//   PauseAt(0, 0);
-//   if(m_pDasherModel != 0) {
-//     m_pDasherModel->Start();
-//   }
-//   if(m_pDasherView != 0) {
-// //     m_pDasherView->ResetSum();
-// //     m_pDasherView->ResetSumCounter();
-// //     m_pDasherView->ResetYAutoOffset();
-//   }
-
-// TODO: Reimplement this sometime
-
-//   int iMinWidth;
-  
-//   if(m_pInputFilter && m_pInputFilter->GetMinWidth(iMinWidth)) {
-//     m_pDasherModel->LimitRoot(iMinWidth);
-//   }
-// }
-
 void CDasherInterfaceBase::PauseAt(int MouseX, int MouseY) {
   SetBoolParameter(BP_DASHER_PAUSED, true);
 
@@ -468,42 +442,46 @@ void CDasherInterfaceBase::PauseAt(int MouseX, int MouseY) {
   Dasher::CStopEvent oEvent;
   m_pEventHandler->InsertEvent(&oEvent);
 
+#ifndef _WIN32_WCE
   if (m_pUserLog != NULL)
     m_pUserLog->StopWriting((float) GetNats());
+#endif
 }
 
-void CDasherInterfaceBase::Halt() {
-  SetBoolParameter(BP_DASHER_PAUSED, true);
-
-  // This will cause us to reinitialise the frame rate counter - ie we start off slowly
-  if(m_pDasherModel != 0)
-    m_pDasherModel->Halt();
+void CDasherInterfaceBase::GameMessageIn(int message, void* messagedata) {
+  GameMode::CDasherGameMode::GetTeacher()->Message(message, messagedata);
 }
 
 void CDasherInterfaceBase::Unpause(unsigned long Time) {
   SetBoolParameter(BP_DASHER_PAUSED, false);
 
-  if(m_pDasherModel != 0) 
+  if(m_pDasherModel != 0)
     m_pDasherModel->Reset_framerate(Time);
 
   Dasher::CStartEvent oEvent;
   m_pEventHandler->InsertEvent(&oEvent);
 
-  ResetNats();
+  // Commenting this out, can't see a good reason to ResetNats,
+  // just because we are not paused anymore - pconlon
+  // ResetNats();
+
+#ifndef _WIN32_WCE
   if (m_pUserLog != NULL)
     m_pUserLog->StartWriting();
+#endif
 }
 
 void CDasherInterfaceBase::CreateInput() {
   if(m_pInput) {
     m_pInput->Deactivate();
-    m_pInput->Unref();
   }
 
   m_pInput = (CDasherInput *)GetModuleByName(GetStringParameter(SP_INPUT_DEVICE));
 
+  if (m_pInput == NULL)
+    m_pInput = (CDasherInput *)GetDefaultInputDevice();
+
   if(m_pInput) {
-    m_pInput->Ref();
     m_pInput->Activate();
   }
 
@@ -512,81 +490,99 @@ void CDasherInterfaceBase::CreateInput() {
 }
 
 void CDasherInterfaceBase::NewFrame(unsigned long iTime, bool bForceRedraw) {
-  // Fail if Dasher is locked
-  if(m_iCurrentState != ST_NORMAL)
+  // Prevent NewFrame from being reentered. This can happen occasionally and
+  // cause crashes.
+  static bool bReentered=false;
+  if (bReentered) {
+#ifdef DEBUG
+    std::cout << "CDasherInterfaceBase::NewFrame was re-entered" << std::endl;
+#endif
     return;
+  }
+  bReentered=true;
+
+  // Fail if Dasher is locked
+  // if(m_iCurrentState != ST_NORMAL)
+  //  return;
 
   bool bChanged(false);
-
+  CExpansionPolicy *pol=m_defaultPolicy;
   if(m_pDasherView != 0) {
     if(!GetBoolParameter(BP_TRAINING)) {
       if (m_pUserLog != NULL) {
-	
+	//ACL note that as of 15/5/09, splitting UpdatePosition into two,
+	//DasherModel no longer guarantees to empty these two if it didn't do anything.
+	//So initialise appropriately...
 	Dasher::VECTOR_SYMBOL_PROB vAdded;
 	int iNumDeleted = 0;
-	
+
 	if(m_pInputFilter) {
-	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel, &vAdded, &iNumDeleted);
+	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel, &vAdded, &iNumDeleted, &pol);
 	}
 
+#ifndef _WIN32_WCE
 	if (iNumDeleted > 0)
 	  m_pUserLog->DeleteSymbols(iNumDeleted);
 	if (vAdded.size() > 0)
 	  m_pUserLog->AddSymbols(&vAdded);
-	
+#endif
+
       }
       else {
 	if(m_pInputFilter) {
-	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel, 0, 0);
+	  bChanged = m_pInputFilter->Timer(iTime, m_pDasherView, m_pDasherModel, 0, 0, &pol);
 	}
       }
-      
+
       m_pDasherModel->CheckForNewRoot(m_pDasherView);
     }
   }
 
+  // Flags at this stage:
+  //
+  // - bChanged = the display was updated, so needs to be rendered to the display
+  // - m_bLastChanged = bChanged was true last time around
+  // - m_bRedrawScheduled = Display invalidated internally
+  // - bForceRedraw = Display invalidated externally
+
   // TODO: This is a bit hacky - we really need to sort out the redraw logic
   if((!bChanged && m_bLastChanged) || m_bRedrawScheduled || bForceRedraw) {
     m_pDasherView->Screen()->SetCaptureBackground(true);
     m_pDasherView->Screen()->SetLoadBackground(true);
   }
 
-  Redraw((bChanged || m_bLastChanged) || m_bRedrawScheduled || bForceRedraw);
+  bForceRedraw |= m_bLastChanged;
+  m_bLastChanged = bChanged; //will also be set in Redraw if any nodes were expanded.
+
+  Redraw(bChanged || m_bRedrawScheduled || bForceRedraw, *pol);
 
-  m_bLastChanged = bChanged;
   m_bRedrawScheduled = false;
 
   // This just passes the time through to the framerate tracker, so we
   // know how often new frames are being drawn.
   if(m_pDasherModel != 0)
     m_pDasherModel->NewFrame(iTime);
+
+  bReentered=false;
 }
 
-void CDasherInterfaceBase::CheckRedraw() {
-  if(m_bRedrawScheduled)
-    Redraw(true);
-
-  m_bRedrawScheduled = false;
-}
-
-/// Full redraw - renders model, decorations and blits onto display
-/// buffer. Not recommended if nothing has changed with the model,
-/// otherwise we're wasting effort.
-
-void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
-
-  if(!m_pDasherView || !m_pDasherModel)
+void CDasherInterfaceBase::Redraw(bool bRedrawNodes, CExpansionPolicy &policy) {
+  // No point continuing if there's nothing to draw on...
+  if(!m_pDasherView)
     return;
-  
-  // TODO: More generic notion of 'layers'?
 
+  // Draw the nodes
   if(bRedrawNodes) {
     m_pDasherView->Screen()->SendMarker(0);
-    m_pDasherModel->RenderToView(m_pDasherView, true);
+	if (m_pDasherModel) m_bLastChanged |= m_pDasherModel->RenderToView(m_pDasherView,policy);
   }
-  
+
+  // Draw the decorations
   m_pDasherView->Screen()->SendMarker(1);
-  
+
+  if(GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher())
+    pTeacher->DrawGameDecorations(m_pDasherView);
+
   bool bDecorationsChanged(false);
 
   if(m_pInputFilter) {
@@ -598,19 +594,19 @@ void CDasherInterfaceBase::Redraw(bool bRedrawNodes) {
   bActionButtonsChanged = DrawActionButtons();
 #endif
 
+  // Only blit the image to the display if something has actually changed
   if(bRedrawNodes || bDecorationsChanged || bActionButtonsChanged)
     m_pDasherView->Display();
 }
 
 void CDasherInterfaceBase::ChangeAlphabet() {
-
   if(GetStringParameter(SP_ALPHABET_ID) == "") {
     SetStringParameter(SP_ALPHABET_ID, m_AlphIO->GetDefault());
     // This will result in ChangeAlphabet() being called again, so
     // exit from the first recursion
     return;
   }
-  
+
   // Send a lock event
 
   WriteTrainFileFull();
@@ -621,10 +617,12 @@ void CDasherInterfaceBase::ChangeAlphabet() {
 
   CreateNCManager();
 
+#ifndef _WIN32_WCE
   // Let our user log object know about the new alphabet since
   // it needs to convert symbols into text for the log file.
   if (m_pUserLog != NULL)
     m_pUserLog->SetAlphabetPtr(m_Alphabet);
+#endif
 
   // Apply options from alphabet
 
@@ -636,12 +634,13 @@ void CDasherInterfaceBase::ChangeAlphabet() {
 void CDasherInterfaceBase::ChangeColours() {
   if(!m_ColourIO || !m_DasherScreen)
     return;
- 
+
   // TODO: Make fuction return a pointer directly
   m_DasherScreen->SetColourScheme(&(m_ColourIO->GetInfo(GetStringParameter(SP_COLOUR_ID))));
 }
 
 void CDasherInterfaceBase::ChangeScreen(CDasherScreen *NewScreen) {
+  // What does ChangeScreen do?
   m_DasherScreen = NewScreen;
   ChangeColours();
 
@@ -652,19 +651,29 @@ void CDasherInterfaceBase::ChangeScreen(CDasherScreen *NewScreen) {
   }
 
   PositionActionButtons();
-  Redraw(true);
+  BudgettingPolicy pol(GetLongParameter(LP_NODE_BUDGET)); //maintain budget, but allow arbitrary amount of work.
+  Redraw(true, pol); // (we're assuming resolution changes are occasional, i.e.
+	// we don't need to worry about maintaining the frame rate, so we can do
+	// as much work as necessary. However, it'd probably be better still to
+  // get a node queue from the input filter, as that might have a different
+  // policy / budget.
 }
 
 void CDasherInterfaceBase::ChangeView() {
   // TODO: Actually respond to LP_VIEW_ID parameter (although there is only one view at the moment)
 
-  if(m_DasherScreen != 0 && m_pDasherModel != 0) {
+  // removed condition that m_pDasherModel != 0. Surely the view can exist without the model?-pconlon
+    if(m_DasherScreen != 0 /*&& m_pDasherModel != 0*/) {
     delete m_pDasherView;
-    
+
     m_pDasherView = new CDasherViewSquare(m_pEventHandler, m_pSettingsStore, m_DasherScreen);
- 
+
     if (m_pInput)
       m_pDasherView->SetInput(m_pInput);
+
+    // Tell the Teacher which view we are using
+    if(GameMode::CDasherGameMode* pTeacher = GameMode::CDasherGameMode::GetTeacher())
+	pTeacher->SetDasherView(m_pDasherView);
   }
 }
 
@@ -680,74 +689,6 @@ void CDasherInterfaceBase::DeleteAlphabet(const std::string &AlphID) {
   m_AlphIO->Delete(AlphID);
 }
 
-/*
-	I've used C style I/O because I found that C++ style I/O bloated
-	the Win32 code enormously. The overhead of loading the buffer into
-	the string instead of reading straight into a string seems to be
-	negligible compared to huge requirements elsewhere.
-*/
-int CDasherInterfaceBase::TrainFile(std::string Filename, int iTotalBytes, int iOffset) {
-  if(Filename == "")
-    return 0;
-  
-  FILE *InputFile;
-  if((InputFile = fopen(Filename.c_str(), "r")) == (FILE *) 0)
-    return 0;
-
-  AddLock(0);
-
-  const int BufferSize = 1024;
-  char InputBuffer[BufferSize];
-  string StringBuffer;
-  int NumberRead;
-  int iTotalRead(0);
-
-  vector < symbol > Symbols;
-
-  CAlphabetManagerFactory::CTrainer * pTrainer = m_pNCManager->GetTrainer();
-  do {
-    NumberRead = fread(InputBuffer, 1, BufferSize - 1, InputFile);
-    InputBuffer[NumberRead] = '\0';
-    StringBuffer += InputBuffer;
-    bool bIsMore = false;
-    if(NumberRead == (BufferSize - 1))
-      bIsMore = true;
-
-    Symbols.clear();
-    m_Alphabet->GetSymbols(&Symbols, &StringBuffer, bIsMore);
-
-    pTrainer->Train(Symbols);
-    iTotalRead += NumberRead;
-  
-    // TODO: No reason for this to be a pointer (other than cut/paste laziness!)
-    CLockEvent *pEvent;
-    pEvent = new CLockEvent("Training Dasher", true, static_cast<int>((100.0 * (iTotalRead + iOffset))/iTotalBytes));
-    m_pEventHandler->InsertEvent(pEvent);
-    delete pEvent;
-
-  } while(NumberRead == BufferSize - 1);
-
-  delete pTrainer;
-
-  fclose(InputFile);
-
-  ReleaseLock(0);
-
-  return iTotalRead;
-}
-
-void CDasherInterfaceBase::GetFontSizes(std::vector <int >*FontSizes) const {
-  FontSizes->push_back(20);
-  FontSizes->push_back(14);
-  FontSizes->push_back(11);
-  FontSizes->push_back(40);
-  FontSizes->push_back(28);
-  FontSizes->push_back(22);
-  FontSizes->push_back(80);
-  FontSizes->push_back(56);
-  FontSizes->push_back(44);
-}
-
 double CDasherInterfaceBase::GetCurCPM() {
   //
   return 0;
@@ -809,7 +750,7 @@ void CDasherInterfaceBase::ResetNats() {
 //  	PauseAt(0,0);
 //        }
 //      }
-    
+
 //      strCurrentContext = strNewContext;
 //      WriteTrainFileFull();
 //    }
@@ -834,7 +775,7 @@ void CDasherInterfaceBase::ResetNats() {
 
 std::string CDasherInterfaceBase::GetContext(int iStart, int iLength) {
   m_strContext = "";
-  
+
   CEditContextEvent oEvent(iStart, iLength);
   m_pEventHandler->InsertEvent(&oEvent);
 
@@ -863,7 +804,7 @@ void CDasherInterfaceBase::SetBoolParameter(int iParameter, bool bValue) {
   m_pSettingsStore->SetBoolParameter(iParameter, bValue);
 };
 
-void CDasherInterfaceBase::SetLongParameter(int iParameter, long lValue) { 
+void CDasherInterfaceBase::SetLongParameter(int iParameter, long lValue) {
   m_pSettingsStore->SetLongParameter(iParameter, lValue);
 };
 
@@ -893,12 +834,12 @@ CUserLogBase* CDasherInterfaceBase::GetUserLogPtr() {
   return m_pUserLog;
 }
 
-void CDasherInterfaceBase::KeyDown(int iTime, int iId) {
+void CDasherInterfaceBase::KeyDown(int iTime, int iId, bool bPos, int iX, int iY) {
   if(m_iCurrentState != ST_NORMAL)
     return;
 
   if(m_pInputFilter && !GetBoolParameter(BP_TRAINING)) {
-    m_pInputFilter->KeyDown(iTime, iId, m_pDasherModel, m_pUserLog);
+    m_pInputFilter->KeyDown(iTime, iId, m_pDasherView, m_pDasherModel, m_pUserLog, bPos, iX, iY);
   }
 
   if(m_pInput && !GetBoolParameter(BP_TRAINING)) {
@@ -906,12 +847,12 @@ void CDasherInterfaceBase::KeyDown(int iTime, int iId) {
   }
 }
 
-void CDasherInterfaceBase::KeyUp(int iTime, int iId) {
+void CDasherInterfaceBase::KeyUp(int iTime, int iId, bool bPos, int iX, int iY) {
   if(m_iCurrentState != ST_NORMAL)
     return;
 
   if(m_pInputFilter && !GetBoolParameter(BP_TRAINING)) {
-    m_pInputFilter->KeyUp(iTime, iId, m_pDasherModel);
+    m_pInputFilter->KeyUp(iTime, iId, m_pDasherView, m_pDasherModel, bPos, iX, iY);
   }
 
   if(m_pInput && !GetBoolParameter(BP_TRAINING)) {
@@ -923,51 +864,71 @@ void CDasherInterfaceBase::CreateInputFilter()
 {
   if(m_pInputFilter) {
     m_pInputFilter->Deactivate();
-    m_pInputFilter->Unref();
     m_pInputFilter = NULL;
   }
 
+#ifndef _WIN32_WCE
   m_pInputFilter = (CInputFilter *)GetModuleByName(GetStringParameter(SP_INPUT_FILTER));
+#endif
 
-  if(m_pInputFilter) {
-    m_pInputFilter->Ref();
-    m_pInputFilter->Activate();
-  }
-  else {
-    // Fall back to a sensible alternative if for some reason the
-    // current choice isn't valid.
-    if(GetStringParameter(SP_INPUT_FILTER) != "Normal Control")
-      SetStringParameter(SP_INPUT_FILTER, "Normal Control");
-  }
+  if (m_pInputFilter == NULL)
+    m_pInputFilter = (CInputFilter *)GetDefaultInputMethod();
+
+  m_pInputFilter->Activate();
 }
 
-void CDasherInterfaceBase::RegisterFactory(CModuleFactory *pFactory) {
-  m_oModuleManager.RegisterFactory(pFactory);
+CDasherModule *CDasherInterfaceBase::RegisterModule(CDasherModule *pModule) {
+    return m_oModuleManager.RegisterModule(pModule);
 }
- 
+
 CDasherModule *CDasherInterfaceBase::GetModule(ModuleID_t iID) {
-   return m_oModuleManager.GetModule(iID);
+    return m_oModuleManager.GetModule(iID);
 }
 
 CDasherModule *CDasherInterfaceBase::GetModuleByName(const std::string &strName) {
-   return m_oModuleManager.GetModuleByName(strName);
+    return m_oModuleManager.GetModuleByName(strName);
 }
 
-void CDasherInterfaceBase::CreateFactories() {
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDefaultFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel,3, _("Normal Control"))));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new COneDimensionalFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel)));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CEyetrackerFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel)));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CClickFilter(m_pEventHandler, m_pSettingsStore, this)));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDynamicFilter(m_pEventHandler, m_pSettingsStore, this)));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CTwoButtonDynamicFilter(m_pEventHandler, m_pSettingsStore, this, 14, 1, _("Two Button Dynamic Mode"))));
-  // TODO: specialist factory for button mode
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 5, 1, true,8, _("Menu Mode"))));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 3, 0, false,10, _("Direct Mode"))));
-  //  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 4, 0, false,11, "Buttons 3")));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 3, 3, false,12, _("Alternating Direct Mode"))));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 4, 2, false,13, _("Compass Mode"))));
-  RegisterFactory(new CWrapperFactory(m_pEventHandler, m_pSettingsStore, new CStylusFilter(m_pEventHandler, m_pSettingsStore, this, m_pDasherModel,15, _("Stylus Control"))));
+CDasherModule *CDasherInterfaceBase::GetDefaultInputDevice() {
+    return m_oModuleManager.GetDefaultInputDevice();
+}
 
+CDasherModule *CDasherInterfaceBase::GetDefaultInputMethod() {
+    return m_oModuleManager.GetDefaultInputMethod();
+}
+
+void CDasherInterfaceBase::SetDefaultInputDevice(CDasherModule *pModule) {
+    m_oModuleManager.SetDefaultInputDevice(pModule);
+}
+
+void CDasherInterfaceBase::SetDefaultInputMethod(CDasherModule *pModule) {
+    m_oModuleManager.SetDefaultInputMethod(pModule);
+}
+
+void CDasherInterfaceBase::CreateModules() {
+  SetDefaultInputMethod(
+    RegisterModule(new CDefaultFilter(m_pEventHandler, m_pSettingsStore, this, 3, _("Normal Control")))
+  );
+  RegisterModule(new COneDimensionalFilter(m_pEventHandler, m_pSettingsStore, this));
+  RegisterModule(new CEyetrackerFilter(m_pEventHandler, m_pSettingsStore, this));
+#ifndef _WIN32_WCE
+  RegisterModule(new CClickFilter(m_pEventHandler, m_pSettingsStore, this));
+#else
+  SetDefaultInputMethod(
+    RegisterModule(new CClickFilter(m_pEventHandler, m_pSettingsStore, this));
+  );
+#endif
+  RegisterModule(new COneButtonFilter(m_pEventHandler, m_pSettingsStore, this));
+  RegisterModule(new COneButtonDynamicFilter(m_pEventHandler, m_pSettingsStore, this));
+  RegisterModule(new CTwoButtonDynamicFilter(m_pEventHandler, m_pSettingsStore, this));
+  RegisterModule(new CTwoPushDynamicFilter(m_pEventHandler, m_pSettingsStore, this));
+  // TODO: specialist factory for button mode
+  RegisterModule(new CButtonMode(m_pEventHandler, m_pSettingsStore, this, true, 8, _("Menu Mode")));
+  RegisterModule(new CButtonMode(m_pEventHandler, m_pSettingsStore, this, false,10, _("Direct Mode")));
+  //  RegisterModule(new CDasherButtons(m_pEventHandler, m_pSettingsStore, this, 4, 0, false,11, "Buttons 3"));
+  RegisterModule(new CAlternatingDirectMode(m_pEventHandler, m_pSettingsStore, this));
+  RegisterModule(new CCompassMode(m_pEventHandler, m_pSettingsStore, this));
+  RegisterModule(new CStylusFilter(m_pEventHandler, m_pSettingsStore, this, 15, _("Stylus Control")));
 }
 
 void CDasherInterfaceBase::GetPermittedValues(int iParameter, std::vector<std::string> &vList) {
@@ -1062,7 +1023,7 @@ void CDasherInterfaceBase::HandleClickUp(int iTime, int iX, int iY) {
   }
 #endif
 
-  KeyUp(iTime, 100);
+  KeyUp(iTime, 100, true, iX, iY);
 }
 
 void CDasherInterfaceBase::HandleClickDown(int iTime, int iX, int iY) {
@@ -1080,7 +1041,7 @@ void CDasherInterfaceBase::HandleClickDown(int iTime, int iX, int iY) {
   }
 #endif
 
-  KeyDown(iTime, 100);
+  KeyDown(iTime, 100, true, iX, iY);
 }
 
 
@@ -1099,13 +1060,6 @@ double CDasherInterfaceBase::GetFramerate() {
     return 0.0;
 }
 
-int CDasherInterfaceBase::GetRenderCount() {
-  if(m_pDasherView)
-    return(m_pDasherView->GetRenderCount());
-  else
-    return 0;
-}
-
 void CDasherInterfaceBase::AddActionButton(const std::string &strName) {
   m_vRightButtons.push_back(new CActionButton(this, strName, false));
 }
@@ -1119,56 +1073,28 @@ void CDasherInterfaceBase::OnUIRealised() {
 
 void CDasherInterfaceBase::ChangeState(ETransition iTransition) {
   static EState iTransitionTable[ST_NUM][TR_NUM] = {
-    {ST_MODEL, ST_UI, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN},
-    {ST_FORBIDDEN, ST_NORMAL, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN},
-    {ST_NORMAL, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN},
-    {ST_FORBIDDEN, ST_FORBIDDEN, ST_LOCKED, ST_FORBIDDEN, ST_SHUTDOWN},
-    {ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN, ST_NORMAL, ST_FORBIDDEN},
-    {ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN}
+    {ST_MODEL, ST_UI, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN},//ST_START
+    {ST_FORBIDDEN, ST_NORMAL, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN},//ST_MODEL
+    {ST_NORMAL, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN},//ST_UI
+    {ST_FORBIDDEN, ST_FORBIDDEN, ST_LOCKED, ST_FORBIDDEN, ST_SHUTDOWN},//ST_NORMAL
+    {ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN, ST_NORMAL, ST_FORBIDDEN},//ST_LOCKED
+    {ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN, ST_FORBIDDEN}//ST_SHUTDOWN
+  //TR_MODEL_INIT, TR_UI_INIT,   TR_LOCK,      TR_UNLOCK,    TR_SHUTDOWN
   };
 
   EState iNewState(iTransitionTable[m_iCurrentState][iTransition]);
 
   if(iNewState != ST_FORBIDDEN) {
-    LeaveState(m_iCurrentState);
-    EnterState(iNewState);
+    if (iNewState == ST_SHUTDOWN) {
+      ShutdownTimer();  
+      WriteTrainFileFull();      
+    }
 
     m_iCurrentState = iNewState;
   }
 }
 
-void CDasherInterfaceBase::LeaveState(EState iState) {
-
-}
-
-void CDasherInterfaceBase::EnterState(EState iState) {
-  switch(iState) {
-  case ST_SHUTDOWN:
-    ShutdownTimer();  
-    WriteTrainFileFull();
-    break;
-  default:
-    // Not handled
-    break;
-  }
-}
-
-void CDasherInterfaceBase::AddLock(int iLockFlags) {
-  if(m_iLockCount == 0)
-    ChangeState(TR_LOCK);
-
-  ++m_iLockCount;
-}
-
-void CDasherInterfaceBase::ReleaseLock(int iLockFlags) {
-  if(m_iLockCount > 0)
-    --m_iLockCount;
-
-  if(m_iLockCount == 0)
-    ChangeState(TR_UNLOCK);
-}
-
-void CDasherInterfaceBase::SetBuffer(int iOffset) { 
+void CDasherInterfaceBase::SetBuffer(int iOffset) {
    CreateModel(iOffset);
 }
 
@@ -1190,7 +1116,17 @@ void CDasherInterfaceBase::SetControlOffset(int iOffset) {
     m_pDasherModel->SetControlOffset(iOffset);
 }
 
-void CDasherInterfaceBase::ClSet(const std::string &strKey, const std::string &strValue) {
+// Returns 0 on success, an error string on failure.
+const char* CDasherInterfaceBase::ClSet(const std::string &strKey, const std::string &strValue) {
   if(m_pSettingsStore)
-    m_pSettingsStore->ClSet(strKey, strValue);
+    return m_pSettingsStore->ClSet(strKey, strValue);
+  return 0;
 }
+
+
+void
+CDasherInterfaceBase::ImportTrainingText(const std::string &strPath) {
+  if(m_pNCManager)
+    m_pNCManager->ImportTrainingText(strPath);
+}
+
