@@ -20,7 +20,9 @@
 
 #include "../Common/Common.h"
 
-#include "DasherNode.h"
+// #include "AlphabetManager.h" - doesnt seem to be required - pconlon
+
+#include "DasherInterfaceBase.h"
 
 using namespace Dasher;
 using namespace Opts;
@@ -35,16 +37,46 @@ using namespace std;
 static char THIS_FILE[] = __FILE__;
 #endif
 #endif
+static int iNumNodes = 0;
+
+int Dasher::currentNumNodeObjects() {return iNumNodes;}
+
+//TODO this used to be inline - should we make it so again?
+CDasherNode::CDasherNode(CDasherNode *pParent, int iLbnd, int iHbnd, SDisplayInfo *pDisplayInfo) {
+  // TODO: Check that these are disabled for debug builds, and that we're not shipping such a build
+  DASHER_ASSERT(iHbnd >= iLbnd);
+  DASHER_ASSERT(pDisplayInfo != NULL);
+	
+  m_pParent = pParent;  
+  if (pParent) {
+    DASHER_ASSERT(!pParent->GetFlag(NF_ALLCHILDREN));
+    pParent->Children().push_back(this);
+  }
+
+  m_iLbnd = iLbnd;
+  m_iHbnd = iHbnd;
+  m_pDisplayInfo = pDisplayInfo;
+  onlyChildRendered = NULL;
+	
+  // Default flags (make a definition somewhere, pass flags to constructor?)
+  m_iFlags = 0;
+	
+  m_iRefCount = 0;
+  m_iNumSymbols = 0;
+  iNumNodes++;
+}
 
 // TODO: put this back to being inlined
 CDasherNode::~CDasherNode() {
+  //  std::cout << "Deleting node: " << this << std::endl;
   // Release any storage that the node manager has allocated,
   // unreference ref counted stuff etc.
-  m_pNodeManager->ClearNode( this );
-
   Delete_children();
 
+  //  std::cout << "done." << std::endl;
+
   delete m_pDisplayInfo;
+  iNumNodes--;
 }
 
 
@@ -66,20 +98,19 @@ void CDasherNode::Trace() const {
    */
 }
 
-bool CDasherNode::NodeIsParent(CDasherNode *oldnode) const {
-  if(oldnode == m_pParent)
-    return true;
-  else
-    return false;
-
+void CDasherNode::GetContext(CDasherInterfaceBase *pInterface, vector<symbol> &vContextSymbols, int iOffset, int iLength) {
+  if (!GetFlag(NF_SEEN)) {
+    DASHER_ASSERT(m_pParent);
+    if (m_pParent) m_pParent->GetContext(pInterface, vContextSymbols, iOffset,iLength);
+  } else {
+    std::string strContext = pInterface->GetContext(iOffset, iLength);
+    pInterface->GetAlphabet()->GetSymbols(vContextSymbols, strContext);
+  }
 }
 
 CDasherNode *const CDasherNode::Get_node_under(int iNormalization, myint miY1, myint miY2, myint miMousex, myint miMousey) {
   myint miRange = miY2 - miY1;
 
-  // TODO: Manipulating flags in a 'get' method?
-  SetFlag(NF_ALIVE, true);
-
   ChildMap::const_iterator i;
   for(i = GetChildren().begin(); i != GetChildren().end(); i++) {
     CDasherNode *pChild = *i;
@@ -106,7 +137,7 @@ void CDasherNode::OrphanChild(CDasherNode *pChild) {
     }
   }
   
-  pChild->SetParent(NULL);
+  pChild->m_pParent=NULL;
 
   Children().clear();
   SetFlag(NF_ALLCHILDREN, false);
@@ -119,8 +150,8 @@ void CDasherNode::DeleteNephews(CDasherNode *pChild) {
 
   ChildMap::iterator i;
   for(i = Children().begin(); i != Children().end(); i++) {
-    if(*i != pChild) {
-      (*i)->Delete_children();
+      if(*i != pChild) {
+	(*i)->Delete_children();
     }
   }
 }
@@ -128,11 +159,20 @@ void CDasherNode::DeleteNephews(CDasherNode *pChild) {
 // TODO: Need to allow for subnodes
 // TODO: Incorporate into above routine
 void CDasherNode::Delete_children() {
+//    CAlphabetManager::SAlphabetData *pParentUserData(static_cast<CAlphabetManager::SAlphabetData *>(m_pUserData));
+
+//    if((GetDisplayInfo()->strDisplayText)[0] == 'e')
+//      std::cout << "ed: " << this << " " << pParentUserData->iContext << " " << pParentUserData->iOffset << std::endl;
+
+//  std::cout << "Start: " << this << std::endl;
+
   ChildMap::iterator i;
   for(i = Children().begin(); i != Children().end(); i++) {
+    //    std::cout << "CNM: " << (*i)->MgrID() << (*i) << " " << (*i)->Parent() << std::endl;
     delete (*i);
   }
   Children().clear();
+  //  std::cout << "NM: " << MgrID() << std::endl;
   SetFlag(NF_ALLCHILDREN, false);
 }
 
@@ -141,27 +181,19 @@ double CDasherNode::GetProb(int iNormalization) {
   return (double) (m_iHbnd - m_iLbnd) / (double) iNormalization;
 }
 
-void CDasherNode::ConvertWithAncestors() {
-  if(GetFlag(NF_CONVERTED))
-    return;
-  
-  SetFlag(NF_CONVERTED, true);
-
-  if(m_pParent)
-    m_pParent->ConvertWithAncestors();
-}
-
 void CDasherNode::SetFlag(int iFlag, bool bValue) {
-  if(bValue)
+
+ if(bValue)
     m_iFlags = m_iFlags | iFlag;
   else
     m_iFlags = m_iFlags & (~iFlag);
-  
-  m_pNodeManager->SetFlag(this, iFlag, bValue);
 }
  
 void CDasherNode::SetParent(CDasherNode *pNewParent) {
+  DASHER_ASSERT(pNewParent);
+  DASHER_ASSERT(!pNewParent->GetFlag(NF_ALLCHILDREN));
   m_pParent = pNewParent;
+  pNewParent->Children().push_back(this);
 }
 
 int CDasherNode::MostProbableChild() {
@@ -177,3 +209,10 @@ int CDasherNode::MostProbableChild() {
   
   return iMax;
 }
+
+bool CDasherNode::GameSearchChildren(string strTargetUtf8Char) {
+  for (ChildMap::iterator i = Children().begin(); i != Children().end(); i++) {
+    if ((*i)->GameSearchNode(strTargetUtf8Char)) return true;
+  }
+  return false;
+}
