@@ -23,33 +23,24 @@
 
 #include "../Common/Common.h"
 #include "../Common/NoClones.h"
+#include "LanguageModelling/LanguageModel.h"
 #include "DasherTypes.h"
-
-#include "NodeManager.h"
-
-// These includes no longer required? - pconlon
-//#include "Alphabet/GroupInfo.h"
-//#include "LanguageModelling/LanguageModel.h"
-
+namespace Dasher {
+  class CDasherNode;
+  class CDasherInterfaceBase;
+};
 #include <deque>
 #include <iostream>
+#include <vector>
 
 // Node flag constants
 #define NF_COMMITTED 1
-#define NF_ACTIVE 2
-#define NF_ALIVE 4
-#define NF_SEEN 8
-#define NF_CONVERTED 16
-#define NF_GAME 32
-#define NF_ALLCHILDREN 64
-#define NF_SUBNODE 128
-#define NF_SUPER 256
-#define NF_END_GAME 512
-
-namespace Dasher {
-  class CDasherNode;
-  class CDasherModel;
-}
+#define NF_SEEN 2
+#define NF_CONVERTED 4
+#define NF_GAME 8
+#define NF_ALLCHILDREN 16
+#define NF_SUPER 32
+#define NF_END_GAME 64
 
 /// \ingroup Model
 /// @{
@@ -75,6 +66,7 @@ class Dasher::CDasherNode:private NoClones {
     bool bVisible;
     std::string strDisplayText;
   };
+  CDasherNode *onlyChildRendered; //cache that only one child was rendered (as it filled the screen)
 
   /// Container type for storing children. Note that it's worth
   /// optimising this as lookup happens a lot
@@ -87,11 +79,11 @@ class Dasher::CDasherNode:private NoClones {
   /// @param iHbnd Upper bound of node within parent
   /// @param pDisplayInfo Struct containing information on how to display the node
   ///
-  inline CDasherNode(CDasherNode *pParent, int iLbnd, int iHbnd, SDisplayInfo *pDisplayInfo);
+  CDasherNode(CDasherNode *pParent, int iLbnd, int iHbnd, SDisplayInfo *pDisplayInfo);
 
   /// @brief Destructor
   ///
-  ~CDasherNode();
+  virtual ~CDasherNode();
 
   void Trace() const;           // diagnostic
 
@@ -108,10 +100,6 @@ class Dasher::CDasherNode:private NoClones {
   /// NF_COMMITTED - Node is 'above' the root, so corresponding symbol
   /// has been added to text box, language model trained etc
   ///
-  /// NF_ACTIVE - Node is a decendent of the root node (TODO: Isnt this everything?)
-  ///
-  /// NF_ALIVE - Node is large enough to be displayed
-  ///
   /// NF_SEEN - Node has already been output
   ///
   /// NF_CONVERTED - Node has been converted (eg Japanese mode)
@@ -120,8 +108,6 @@ class Dasher::CDasherNode:private NoClones {
   ///
   /// NF_ALLCHILDREN - Node has all children (TODO: obsolete?)
   ///
-  /// NF_SUBNODE - Node should be considered an integral subnode of parents (eg groups)
-  ///
   /// NF_SUPER - Node covers entire visible area
   ///
   /// NF_END_GAME - Node is the last one of the phrase in game mode
@@ -130,7 +116,7 @@ class Dasher::CDasherNode:private NoClones {
   /// @param iFlag The flag to set
   /// @param bValue The new value of the flag
   ///
-  void SetFlag(int iFlag, bool bValue);
+  virtual void SetFlag(int iFlag, bool bValue);
 
   /// @brief Get the value of a flag for this node
   ///
@@ -140,16 +126,6 @@ class Dasher::CDasherNode:private NoClones {
   ///
   inline bool GetFlag(int iFlag) const;
 
-  /// @brief Recursively mark nodes as converted 
-  ///
-  /// Ensure that this node is marked as being converted, together with
-  /// all of its ancestors (assuming that unconverted nodes are
-  /// 'contiguous' at the brances of the tree).
-  ///
-  /// @todo replace with a generic 'recursive set flag'?
-  ///
-  void ConvertWithAncestors();
-
   /// @}
 
   /// @name Routines relating to the size of the node
@@ -205,12 +181,10 @@ class Dasher::CDasherNode:private NoClones {
   /// @name Routines for manipulating relatives
   /// @{
 
-  inline ChildMap & Children();
   inline const ChildMap & GetChildren() const;
   inline unsigned int ChildCount() const;
   inline CDasherNode *Parent() const;
   void SetParent(CDasherNode *pNewParent); 
-  bool NodeIsParent(CDasherNode * oldnode) const;
   // TODO: Should this be here?
   CDasherNode *const Get_node_under(int, myint y1, myint y2, myint smousex, myint smousey);   // find node under given co-ords
   
@@ -233,25 +207,76 @@ class Dasher::CDasherNode:private NoClones {
   ///
   void Delete_children();
   /// @}
-
-  /// \todo Make private, read only access?
-  // leave public, or implement a get method - pconlon
-  CNodeManager *m_pNodeManager;
-
-  /// Pointer for the node manager to do with as it sees fit.
-  /// Remember to make sure that the node manager deletes anything it
-  /// puts here before the node is destroyed.
+  
   ///
-  /// Please put everything in here which isn't required to simply
-  /// render the node - basically the only exceptions should be the
-  /// node size, colour and display text.
-  /// \todo Make private, read only access?
-  void *m_pUserData;
+  /// Sees if a *child* / descendant of the specified node (not that node itself)
+  /// represents the specified character. If so, set the child & intervening nodes'
+  /// NF_GAME flag, and return true; otherwise, return false.
+  ///
+  bool GameSearchChildren(std::string strTargetUtf8Char);
+  
+  /// @name Management routines (once accessed via NodeManager)
+  /// @{
+  virtual int mgrId() = 0;
+  ///
+  /// Provide children for the supplied node
+  ///
+  
+  virtual void PopulateChildren() = 0;
+  
+  /// The number of children which a call to PopulateChildren can be expected to generate.
+  /// (This is not required to be 100% accurate, but any discrepancies will likely cause
+  /// the node budgetting algorithm to behave sub-optimally)
+  virtual int ExpectedNumChildren() = 0;
+    
+  ///
+  /// Called whenever a node belonging to this manager first 
+  /// moves under the crosshair
+  ///
+  
+  virtual void Output(Dasher::VECTOR_SYMBOL_PROB* pAdded, int iNormalization) {};
+  virtual void Undo() {};
+  
+  virtual void Enter() {};
+  virtual void Leave() {};
+  
+  virtual CDasherNode *RebuildParent() {
+    return 0;
+  };
+  
+  ///
+  /// Get as many symbols of context, up to the _end_ of the specified range,
+  /// as possible from this node and its uncommitted ancestors
+  ///
+  virtual void GetContext(CDasherInterfaceBase *pInterface, std::vector<symbol> &vContextSymbols, int iOffset, int iLength);
+  
+  virtual void SetControlOffset(int iOffset) {};
+  
+  ///
+  /// See if this node represents the specified alphanumeric character; if so, set it's NF_GAME flag and
+  /// return true; otherwise, return false.
+  ///
+  virtual bool GameSearchNode(std::string strTargetUtf8Char) {return false;}
+  
+  /// Clone the context of the specified node, if it's an alphabet node;
+  /// else return an empty context. (Used by ConversionManager)
+  virtual CLanguageModel::Context CloneAlphContext(CLanguageModel *pLanguageModel) {
+    return pLanguageModel->CreateEmptyContext();
+  };
+
+  virtual symbol GetAlphSymbol() {
+    throw "Hack for pre-MandarinDasher ConversionManager::BuildTree method, needs to access CAlphabetManager-private struct";
+  }
+
+  /// @}
+  int m_iOffset;
 
   // A hack, to allow this node to be tied to a particular number of symbols;
   int m_iNumSymbols;
   
  private:
+  inline ChildMap &Children();
+
   SDisplayInfo *m_pDisplayInfo;
 
   int m_iLbnd;
@@ -267,32 +292,17 @@ class Dasher::CDasherNode:private NoClones {
 };
 /// @}
 
+namespace Dasher {
+  /// Return the number of CDasherNode objects currently in existence.
+  int currentNumNodeObjects();
+}
+
+
 /////////////////////////////////////////////////////////////////////////////
 // Inline functions
 /////////////////////////////////////////////////////////////////////////////
 
-// Why are these using-directives in a header file?
-using namespace Dasher;
-using namespace Opts;
-// TODO: move to the top of file.
-#include "DasherModel.h"
-
-inline CDasherNode::CDasherNode(CDasherNode *pParent, int iLbnd, int iHbnd, SDisplayInfo *pDisplayInfo) {
-  // TODO: Check that these are disabled for debug builds, and that we're not shipping such a build
-  DASHER_ASSERT(iHbnd >= iLbnd);
-  DASHER_ASSERT(pDisplayInfo != NULL);
-
-  m_pParent = pParent;  
-  m_iLbnd = iLbnd;
-  m_iHbnd = iHbnd;
-  m_pDisplayInfo = pDisplayInfo;
-
-  // Default flags (make a definition somewhere, pass flags to constructor)
-  m_iFlags = NF_ACTIVE | NF_ALIVE;
-
-  m_iRefCount = 0;
-  m_iNumSymbols = 0;
-}
+namespace Dasher {
 
 inline const CDasherNode::SDisplayInfo *CDasherNode::GetDisplayInfo() const {
   return m_pDisplayInfo;
@@ -334,6 +344,5 @@ inline void CDasherNode::SetRange(int iLower, int iUpper) {
   m_iLbnd = iLower;
   m_iHbnd = iUpper;
 }
-  
-
+}
 #endif /* #ifndef __DasherNode_h__ */
