@@ -1,22 +1,32 @@
 // DasherView.cpp
 //
-/////////////////////////////////////////////////////////////////////////////
+// Copyright (c) 2008 The Dasher Team
 //
-// Copyright (c) 2001-2005 David Ward
+// This file is part of Dasher.
 //
-/////////////////////////////////////////////////////////////////////////////
+// Dasher is free software; you can redistribute it and/or modify
+// it under the terms of the GNU General Public License as published by
+// the Free Software Foundation; either version 2 of the License, or
+// (at your option) any later version.
+//
+// Dasher is distributed in the hope that it will be useful,
+// but WITHOUT ANY WARRANTY; without even the implied warranty of
+// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+// GNU General Public License for more details.
+//
+// You should have received a copy of the GNU General Public License
+// along with Dasher; if not, write to the Free Software 
+// Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 
 #include "../Common/Common.h"
 
+#include "DasherGameMode.h"
+#include "DasherInput.h"
+#include "DasherModel.h"
 #include "DasherView.h"
 #include "Event.h"
 #include "EventHandler.h"
-#include "DasherModel.h"
-#include "DasherInput.h"
-
-//#include "DasherGameMode.h"
-
-#include <iostream>
+#include "DasherScreen.h"
 
 using namespace Dasher;
 
@@ -37,20 +47,6 @@ CDasherView::CDasherView(CEventHandler *pEventHandler, CSettingsStore *pSettings
    m_bDemoMode(false), m_bGameMode(false) {
 }
 
-void CDasherView::HandleEvent(Dasher::CEvent *pEvent) {
-  if(pEvent->m_iEventType == 1) {
-    Dasher::CParameterNotificationEvent * pEvt(static_cast < Dasher::CParameterNotificationEvent * >(pEvent));
-    
-    switch (pEvt->m_iParameter) {
-        case BP_DASHER_PAUSED:  
-	
-                  break;    
-	default:
-      break;
-    }
-  }
-};
-
 /////////////////////////////////////////////////////////////////////////////
 
 void CDasherView::ChangeScreen(CDasherScreen *NewScreen) {
@@ -59,13 +55,11 @@ void CDasherView::ChangeScreen(CDasherScreen *NewScreen) {
 
 /////////////////////////////////////////////////////////////////////////////
 
-bool CDasherView::Render(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, bool bRedrawDisplay, std::vector<std::pair<myint,bool> >* pvGameTargetY) {
+void CDasherView::Render(CDasherNode *pRoot, myint iRootMin, myint iRootMax, CExpansionPolicy &policy, bool bRedrawDisplay, std::vector<std::pair<myint,bool> >* pvGameTargetY) {
 
   m_iRenderCount = 0;
   Screen()->SetLoadBackground(false);
-  RenderNodes(pRoot, iRootMin, iRootMax, vNodeList, vDeleteList, pvGameTargetY);
-
-  return true;
+  RenderNodes(pRoot, iRootMin, iRootMax, policy, pvGameTargetY);
 }
 
 int CDasherView::GetCoordinateCount() {
@@ -130,15 +124,15 @@ void CDasherView::DasherPolyline(myint *x, myint *y, int n, int iWidth, int iCol
 }
 
 // Draw a polyline with an arrow on the end
-void CDasherView::DasherPolyarrow(myint *x, myint *y, int n, int iWidth, int iColour) {
+void CDasherView::DasherPolyarrow(myint *x, myint *y, int n, int iWidth, int iColour, double dArrowSizeFactor) {
 
   CDasherScreen::point * ScreenPoints = new CDasherScreen::point[n+3];
 
   for(int i(0); i < n; ++i)
     Dasher2Screen(x[i], y[i], ScreenPoints[i].x, ScreenPoints[i].y);
 
-  int iXvec = (ScreenPoints[n-2].x - ScreenPoints[n-1].x)*1.414;
-  int iYvec = (ScreenPoints[n-2].y - ScreenPoints[n-1].y)*1.414;
+  int iXvec = (int)((ScreenPoints[n-2].x - ScreenPoints[n-1].x)*dArrowSizeFactor);
+  int iYvec = (int)((ScreenPoints[n-2].y - ScreenPoints[n-1].y)*dArrowSizeFactor);
 
   ScreenPoints[n].x   = ScreenPoints[n-1].x + iXvec + iYvec;
   ScreenPoints[n].y   = ScreenPoints[n-1].y - iXvec + iYvec;
@@ -303,8 +297,8 @@ void CDasherView::DasherDrawText(myint iAnchorX1, myint iAnchorY1, myint iAnchor
     myint iDasherNewRight;
     myint iDasherNewBottom;
     
-    Screen2Dasher(newleft2, newtop2, iDasherNewLeft, iDasherNewTop,false,true);
-    Screen2Dasher(newright2, newbottom2, iDasherNewRight, iDasherNewBottom,false,true);
+    Screen2Dasher(newleft2, newtop2, iDasherNewLeft, iDasherNewTop);
+    Screen2Dasher(newright2, newbottom2, iDasherNewRight, iDasherNewBottom);
     
     mostleft = std::min(iDasherNewRight, iDasherNewLeft);
   }
@@ -356,7 +350,7 @@ int CDasherView::GetCoordinates(myint &iDasherX, myint &iDasherY) {
 
   switch(iType) {
   case 0:
-    Screen2Dasher(mousex, mousey, iDasherX, iDasherY, false, true );
+    Screen2Dasher(mousex, mousey, iDasherX, iDasherY);
     break;
   case 1:
     iDasherX = mousex;
@@ -367,18 +361,14 @@ int CDasherView::GetCoordinates(myint &iDasherX, myint &iDasherY) {
     break;
   }
 
-#ifndef WITH_MAEMO
-  // FIXME
-  //  iDasherX = myint(xmap(iDasherX / static_cast < double >(GetLongParameter(LP_MAX_Y))) * GetLongParameter(LP_MAX_Y));
-  // iDasherY = m_ymap.map(iDasherY);
-#endif
   ///GAME///
-//   if(m_bGameMode)
-//     {
-//       if(m_bDemoMode)
-// 	CDasherGameMode::GetTeacher()->DemoModeGetCoordinates(iDasherX, iDasherY);
-//       CDasherGameMode::GetTeacher()->SetUserMouseCoordinates(iDasherX, iDasherY);
-//     }
+  if(m_bGameMode)
+    {
+      using GameMode::CDasherGameMode;
+      if(m_bDemoMode)
+        CDasherGameMode::GetTeacher()->DemoModeGetCoordinates(iDasherX, iDasherY);
+      CDasherGameMode::GetTeacher()->SetUserMouseCoordinates(iDasherX, iDasherY);
+    }
       
   return iType;
 }
