@@ -17,6 +17,7 @@ namespace Dasher {
 #include "DasherTypes.h"
 #include "DasherComponent.h"
 #include "View/DelayedDraw.h"
+#include "ExpansionPolicy.h"
 
 /// \defgroup View Visualisation of the model
 /// @{
@@ -61,11 +62,6 @@ public:
 
   virtual ~CDasherView() {
   }
-  ///
-  /// Event handler
-  /// \param pEvent Pointer to incoming event
-  virtual void HandleEvent(Dasher::CEvent * pEvent);
-
 
   /// @name Pointing device mappings 
   /// @{
@@ -96,7 +92,7 @@ public:
   /// Convert a screen co-ordinate to Dasher co-ordinates
   ///
 
-  virtual void Screen2Dasher(screenint iInputX, screenint iInputY, myint & iDasherX, myint & iDasherY,bool b1D, bool bNonlinearity) = 0;
+  virtual void Screen2Dasher(screenint iInputX, screenint iInputY, myint & iDasherX, myint & iDasherY) = 0;
 
   ///
   /// Convert Dasher co-ordinates to screen co-ordinates
@@ -104,21 +100,15 @@ public:
 
   virtual void Dasher2Screen(myint iDasherX, myint iDasherY, screenint & iScreenX, screenint & iScreenY) = 0;
 
+  ///
+  /// Convert Dasher co-ordinates to polar co-ordinates (r,theta), with 0<r<1, 0<theta<2*pi
+  ///
+  virtual void Dasher2Polar(myint iDasherX, myint iDasherY, double &r, double &theta) = 0;
 
   virtual bool IsNodeVisible(myint y1, myint y2) { return true; };
 
   virtual void VisibleRegion( myint &iDasherMinX, myint &iDasherMinY, myint &iDasherMaxX, myint &iDasherMaxY ) = 0;
 
-  /// \todo This function is only public (and in the parent class)
-  /// because of the slightly hacky conversion needed for auto speed
-  /// control. At some point find a way to make this more sensible.
-  virtual double xmap(double x) const = 0;
-
-  /// \todo This function is only public (and in the parent class)
-  /// because of the slightly hacky conversion needed for auto speed
-  /// control. At some point find a way to make this more sensible.
-  virtual double ymap(double x) const = 0;
-
   /// @}
 
   /// Change the screen - must be called if the Screen is replaced or resized
@@ -130,15 +120,9 @@ public:
   /// Drawing more complex structures, generally implemented by derived class
   /// @{
 
-  void RenderModel(CDasherModel *pModel);
-
   /// Renders Dasher with mouse-dependent items
   /// \todo Clarify relationship between Render functions and probably only expose one
-  virtual bool Render(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, bool bRedrawDisplay, std::vector<std::pair<myint,bool> >* pvGameTargetY);
-
-  int GetRenderCount() {
-    return m_iRenderCount;
-  };
+  virtual void Render(CDasherNode *pRoot, myint iRootMin, myint iRootMax, CExpansionPolicy &policy, bool bRedrawDisplay, std::vector<std::pair<myint,bool> >* pvGameTargetY);
 
   /// @}
 
@@ -209,7 +193,7 @@ private:
   CDasherInput *m_pInput;       // Input device abstraction
 
   /// Renders the Dasher node structure
-  virtual void RenderNodes(CDasherNode *pRoot, myint iRootMin, myint iRootMax, std::vector<CDasherNode *> &vNodeList, std::vector<CDasherNode *> &vDeleteList, std::vector<std::pair<myint,bool> > *pvGamePointer) = 0;
+  virtual void RenderNodes(CDasherNode *pRoot, myint iRootMin, myint iRootMax, CExpansionPolicy &policy, std::vector<std::pair<myint,bool> > *pvGamePointer) = 0;
 
 
   /// Get the co-ordinates from the input device
@@ -219,6 +203,5 @@ private:
   bool m_bGameMode;
 };
 /// @}
-#include "DasherView.inl"
 
 #endif /* #ifndef __DasherView_h_ */
