@@ -8,10 +8,12 @@
 
 #include <iostream>
 
-CDefaultFilter::CDefaultFilter(Dasher::CEventHandler * pEventHandler, CSettingsStore *pSettingsStore, CDasherInterfaceBase *pInterface, CDasherModel *m_pDasherModel, ModuleID_t iID, const char *szName)
+using namespace Dasher;
+
+CDefaultFilter::CDefaultFilter(Dasher::CEventHandler * pEventHandler, CSettingsStore *pSettingsStore, CDasherInterfaceBase *pInterface, ModuleID_t iID, const char *szName)
   : CInputFilter(pEventHandler, pSettingsStore, pInterface, iID, 1, szName) {
   m_pStartHandler = 0;
-  m_pAutoSpeedControl = new CAutoSpeedControl(m_pEventHandler, m_pSettingsStore, m_pDasherModel->Framerate());
+  m_pAutoSpeedControl = new CAutoSpeedControl(m_pEventHandler, m_pSettingsStore);
 
   CreateStartHandler();
 }
@@ -40,31 +42,36 @@ bool CDefaultFilter::DecorateView(CDasherView *pView) {
   return bDidSomething;
 }
 
-bool CDefaultFilter::Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted) {
-  myint iDasherX;
-  myint iDasherY;
+bool CDefaultFilter::Timer(int Time, CDasherView *m_pDasherView, CDasherModel *m_pDasherModel, Dasher::VECTOR_SYMBOL_PROB *pAdded, int *pNumDeleted, CExpansionPolicy **pol) {
+  bool bDidSomething = false;
+  if (!GetBoolParameter(BP_DASHER_PAUSED))
+  {
+    myint iDasherX;
+    myint iDasherY;
 
-  m_pDasherView->GetCoordinates(iDasherX, iDasherY);
+    m_pDasherView->GetCoordinates(iDasherX, iDasherY);
 
-  ApplyAutoCalibration(iDasherX, iDasherY, true);
-  ApplyTransform(iDasherX, iDasherY);
+    ApplyAutoCalibration(iDasherX, iDasherY, true);
+    ApplyTransform(iDasherX, iDasherY);
 
-  if(GetBoolParameter(BP_PAUSE_OUTSIDE) && !GetBoolParameter(BP_DASHER_PAUSED)) {
-    myint iDasherMinX;
-    myint iDasherMinY;
-    myint iDasherMaxX;
-    myint iDasherMaxY;
-    m_pDasherView->VisibleRegion(iDasherMinX, iDasherMinY, iDasherMaxX, iDasherMaxY);
+    if(GetBoolParameter(BP_PAUSE_OUTSIDE)) {
+      myint iDasherMinX;
+      myint iDasherMinY;
+      myint iDasherMaxX;
+      myint iDasherMaxY;
+      m_pDasherView->VisibleRegion(iDasherMinX, iDasherMinY, iDasherMaxX, iDasherMaxY);
   
-    if((iDasherX > iDasherMaxX) || (iDasherX < iDasherMinX) || (iDasherY > iDasherMaxY) || (iDasherY < iDasherMinY))
-      m_pInterface->PauseAt(0,0);
+      if((iDasherX > iDasherMaxX) || (iDasherX < iDasherMinX) || (iDasherY > iDasherMaxY) || (iDasherY < iDasherMinY))
+        m_pInterface->PauseAt(0,0);
+		return false;
+    }
+
+    m_pDasherModel->OneStepTowards(iDasherX,iDasherY, Time, pAdded, pNumDeleted);
+    bDidSomething = true;
+
+    m_pAutoSpeedControl->SpeedControl(iDasherX, iDasherY, m_pDasherModel->Framerate(), m_pDasherView);
   }
-
-  bool bDidSomething;
-  bDidSomething = m_pDasherModel->UpdatePosition(iDasherX,iDasherY, Time, pAdded, pNumDeleted);
-
-  m_pAutoSpeedControl->SpeedControl(iDasherX, iDasherY, m_pDasherModel->Framerate(), m_pDasherView);
-
+	
   if(m_pStartHandler)
     m_pStartHandler->Timer(Time, m_pDasherView, m_pDasherModel);
 
@@ -131,7 +138,7 @@ void CDefaultFilter::DrawMouse(CDasherView *pView) {
   ApplyAutoCalibration(iDasherX, iDasherY, false);
   ApplyTransform(iDasherX, iDasherY);
 
-  pView->DasherDrawCentredRectangle(iDasherX, iDasherY, 5, 2, Opts::ColorSchemes(Objects), false);
+  pView->DasherDrawCentredRectangle(iDasherX, iDasherY, 5, 2, Opts::ColorSchemes(Opts::Objects), false);
 }
 
 void CDefaultFilter::DrawMouseLine(CDasherView *pView) {
