@@ -9,6 +9,8 @@
 #ifndef __LanguageModelling_LanguageModel_h__
 #define __LanguageModelling_LanguageModel_h__
 
+#include "../DasherTypes.h"
+
 #include "SymbolAlphabet.h"
 #include "../DasherComponent.h"
 #include <vector>
@@ -54,7 +56,7 @@ public:
 
   ///
   /// @name Context manipulation
-  /// Functions for creating, destroying and alteringn contexts
+  /// Functions for creating, destroying and altering contexts
   /// @{
 
   ///
@@ -102,20 +104,6 @@ public:
 
   /// @}
 
-  /// @name Status reporting
-  /// Return information usful for debgging purposes
-  /// @{
-
-  /// Get some measure of the memory usage for diagnostic
-  /// purposes. No need to implement this if you're not comparing
-  /// language models. The exact meaning of the result will
-  /// depend on the implementation (for example, could be the
-  /// number of nodes in a trie, or the physical memory usage).
-
-  virtual int GetMemory() = 0;
-
-  /// @}
-
   /// @name Persistant storage
   /// Binary representation of language model state
   /// @{
@@ -149,11 +137,6 @@ public:
     return 5;
   };
 
-  symbol GetResetSymbol() const {
-    return m_Alphabet.GetAlphabetPointer()->GetResetSymbol();
-  };
-
-
  protected:
   struct SLMFileHeader {
     // Magic number ("%DLF" in ASCII)
