@@ -9,6 +9,8 @@
 #include "../../Common/Common.h"
 #include "WordLanguageModel.h"
 #include "PPMLanguageModel.h"
+#include "../Alphabet/Alphabet.h"
+
 
 #include <cmath>
 #include <cstdlib>
@@ -164,7 +166,7 @@ CWordLanguageModel::CWordLanguageModel(Dasher::CEventHandler *pEventHandler, CSe
       //      std::cout << SymbolAlphabet().GetAlphabetPointer() << std::endl;
 
       std::vector < symbol > Symbols;
-      SymbolAlphabet().GetAlphabetPointer()->GetSymbols(&Symbols, &CurrentWord, false);
+      SymbolAlphabet().GetAlphabetPointer()->GetSymbols(Symbols, CurrentWord);
 
       for(std::vector < symbol >::iterator it(Symbols.begin()); it != Symbols.end(); ++it) {
         pSpellingModel->LearnSymbol(TempContext, *it);
