@@ -17,9 +17,12 @@
 #include "../../DasherCore/DasherInterfaceBase.h"
 #include "../../DasherCore/ColourIO.h"
 
-class CAdvancedPage : public CPrefsPageBase {
+class CAdvancedPage:public CPrefsPageBase {
 public:
-  CAdvancedPage(HWND Parent, CDasherInterfaceBase * DI, CAppSettings *pAppSettings);
+  CAdvancedPage(HWND Parent, Dasher::CDasherInterfaceBase * DI, CAppSettings *pAppSettings);
+  
+  LRESULT WndProc(HWND Window, UINT message, WPARAM wParam, LPARAM lParam);
+
 protected:
 private:
   HWND CustomBox;
@@ -35,7 +38,6 @@ private:
   void InitCustomBox();
   bool UpdateInfo();
   bool Apply();
-  bool Validate();
 
  TCHAR m_tcBuffer[1000];
 
