@@ -12,7 +12,7 @@ extern CONST UINT DASHER_SHOW_PREFS;
 
 class CStatusControl : public ATL::CWindowImpl<CStatusControl> {
 public:
-  CStatusControl(CDasherInterfaceBase *pDasherInterface);
+	CStatusControl(Dasher::CDasherInterfaceBase *pDasherInterface);
 
   // ATL boilerplate code
   DECLARE_WND_SUPERCLASS(L"STATUSCONTROL", L"STATIC");
@@ -31,6 +31,10 @@ public:
   // Create the window, with children
   HWND Create(HWND hParent);
 
+  int GetHeight() {
+    return m_iEditHeight + 2;
+  }
+
 private:
   // Create the child windows of the control
   void CreateChildren();
@@ -54,7 +58,7 @@ private:
   void UpdateSpeed(int iPos, int iDelta);
 
   // The Dasher interface with which this control communicates
-  CDasherInterfaceBase *m_pDasherInterface;
+  Dasher::CDasherInterfaceBase *m_pDasherInterface;
 
   // Handles to child windows
   HWND m_hEdit;
@@ -64,6 +68,7 @@ private:
   HWND m_hAlphabetLabel;
 
   int m_iEditWidth;
+  int m_iEditHeight;
 };
 
 #endif
\ No newline at end of file
