@@ -19,8 +19,9 @@
 
 class CViewPage:public CPrefsPageBase {
 public:
-  CViewPage(HWND Parent, CDasherInterfaceBase *DI, CAppSettings *pAppSettings);
-protected:
+	CViewPage(HWND Parent, Dasher::CDasherInterfaceBase *DI, CAppSettings *pAppSettings);
+  LRESULT WndProc(HWND Window, UINT message, WPARAM wParam, LPARAM lParam);
+
 private:
   HWND CustomBox;
 
@@ -32,8 +33,8 @@ private:
   void PopulateList();
   void InitCustomBox();
   bool UpdateInfo();
- bool Apply();
- bool Validate();
+  bool Apply();
+
 };
 
 #endif  /* #ifndef */
